/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package kelp.webapp.presentation;

// XMLC imports
import org.enhydra.xml.xmlc.XMLCUtil;

// Xerces imports
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.html.HTMLTableElement;
import org.w3c.dom.html.HTMLTableRowElement;

// Servlet imports
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

// Standard imports
import java.io.IOException;

/**
 * <p>
 * This presentation object dynamically creates an HTML page
 * showing the greeting from the application configuration file.
 * </p>
 */
public class TableServlet extends HttpServlet {
    public void doGet(HttpServletRequest request,
                      HttpServletResponse response) throws ServletException,
                      IOException {

        TableHTML page = null;
        ServletOutputStream out;
        byte[] buffer;

        page = createPage();
        buffer = page.toDocument().getBytes();
        response.setContentType( "text/html" );
        response.setContentLength( buffer.length );
        out = response.getOutputStream();
        out.write(buffer);
        out.flush();
        response.flushBuffer();

    }

    public TableHTML createPage() {
        String[]            names = {
            "Travis Shook", "Kelly Rose", "Lloyd Tabb", "David Toth",
            "Adam Menkes", "Randy Solton"
        };
        TableHTML           page = null;
        HTMLTableRowElement newRow = null;
        HTMLTableRowElement dataRow = null;
        HTMLTableElement    table = null;

        page = new TableHTML();
        table = page.getElementSampleTable();
        dataRow = page.getElementDataRow();
        clearTable(table, dataRow);
        for (int i = 0; i < names.length; i++) {
            newRow = createNewRow(dataRow, names[i], i);
            table.appendChild(newRow);
        }
        return page;
    }

    private HTMLTableRowElement createNewRow(HTMLTableRowElement dataRow,
                                             String newData, int i) {
        HTMLTableRowElement rowNew = null;
        Element             dataCell = null;

        rowNew = (HTMLTableRowElement) dataRow.cloneNode(true);
        dataCell = XMLCUtil.getRequiredElementById("cellOne", rowNew);
        XMLCUtil.getFirstText(dataCell).setData("" + (i + 1));
        dataCell = XMLCUtil.getRequiredElementById("cellTwo", rowNew);
        XMLCUtil.getFirstText(dataCell).setData(newData);
        return rowNew;
    }

    /**
     * Delete all table elements that do not have IDs and
     * also delete data row.
     */
    protected void clearTable(Node node, HTMLTableRowElement dataRow) {
        if (node instanceof HTMLTableRowElement) {
            String id = ((HTMLTableRowElement) node).getId();

            if ((id == null) || (id.length() == 0) || (node == dataRow)) {
                node.getParentNode().removeChild(node);
            }
        } else {

            // Not a row, search children.
            Node child = node.getFirstChild();

            while (child != null) {
                Node next = child.getNextSibling();

                clearTable(child, dataRow);
                child = next;
            }
        }
    }

}
