/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.codegen;

// ToolBox imports
import org.enhydra.tool.codegen.ProjectGenerator;
import org.enhydra.tool.codegen.GeneratorException;
import org.enhydra.tool.codegen.internal.ServiceOptionSet;
import org.enhydra.tool.codegen.internal.ServiceGenerator;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterJavaFileNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;

// Standard imports
import java.awt.Component;
import java.io.File;
import java.io.FilenameFilter;
import java.util.Vector;
import java.util.ResourceBundle;

//
public class ServiceUtil extends ProjectGenUtil {

    // string not to be resourced
    private final static String DEFAULT_COMMAND_NAME = "service";    // nores

    //
    public ServiceUtil() throws CodeGenException {
        super(new ServiceGenerator());
    }

    public ServiceUtil(OtterProject project) throws CodeGenException {
        super(project, new ServiceGenerator());
    }

    public ServiceUtil(OtterProject project, ServiceGenerator gen) throws CodeGenException {
        super(project, gen);
    }


    public static ProjectGenerator createGenerator() {
        return new ServiceGenerator();
    }

    public static String getCommandName() {
        String commandName = DEFAULT_COMMAND_NAME;

        if (KelpInfo.isClassPathComplete()) {
            commandName = ServiceGenerator.COMMAND_NAME;
        }
        return commandName;
    }

    /**
     * Method declaration
     *
     * @return
     */
    public static String getDisplayName() {
        String displayName = res.getString("Web_Application");

        if (KelpInfo.isClassPathComplete()) {
            displayName = ServiceGenerator.displayName;
        }
        return displayName;
    }

    // override ProjectGenUtil
    protected void initOptions() throws GeneratorException {
    	String serviceName = new String();
    	StringBuffer buf = new StringBuffer();
    	super.initOptions();

        serviceName = getGenerator().getOptionSet().lookup(ServiceOptionSet.PROJECT).getValue();
        buf.append(Character.toUpperCase(serviceName.charAt(0)));
        if (serviceName.length() > 1) {
          buf.append(serviceName.substring(1));
        }
        serviceName = buf.toString();
        getGenerator().getOptionSet().lookup(ServiceOptionSet.SERVICE).setValue(serviceName);
        getGenerator().getOptionSet().lookup(ServiceOptionSet.MBEAN).setValue(true);
    }

}
