/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.deployer;

// AddinCore
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.event.WriteEvent;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.swing.OutputPanel;

// ToolBox
import org.enhydra.tool.common.DataValidationException;
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;

// JDK
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.PrintWriter;
import java.lang.ref.WeakReference;
import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.beans.*;

//
public class GeneralPanel extends JPanel implements Instructor, DeployStep {
    private GridBagLayout       layoutMain;
    private JCheckBox           checkDeployBuild;
    private JPanel              panelDeployRoot;
    private GridBagLayout       layoutDeployRoot;
    private JTextField          textDeployRoot;
    private JButton             buttonDeployRoot;
    private JPanel              panelMessage;
    private JPanel              panelFiller;
    private JCheckBox           checkWriteMessage;
    private JTextField          textOutputFilename;
    private JButton             buttonMessage;
    private JCheckBox           checkEcho;
    private JCheckBox           checkOverwrite;
    private JTabbedPane         tabMessage;
    private JPanel              panelMessageOption;
    private OutputPanel         outputPanel;
    private GridBagLayout       layoutMessageOption;
    private BorderLayout        layoutMessage;
    private LocalButtonListener buttonListener = null;
    private LocalCheckListener  checkListener = null;
    private LocalComboListener  comboListener = null;
    private boolean             ran = false;
    private JPanel              panelType;
    private JComboBox           comboType;
    private WeakReference       projectRef = null;
    private GridBagLayout       layoutType;

    public GeneralPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // implements Instructor
    public String getTab() {
        return "General";
    }

    // implements Instructor
    public String getTitle() {
        return "General deployment options";
    }

    // implements Instructor
    public String getInstructions() {
        return "Messaging options control how status messages are handled during deployment.";
    }

    // implements DeployStep
    public void refresh() {

        // nothing to refresh
    }

    // implements DeployStep
    public void clearAll() {
        buttonDeployRoot.removeActionListener(buttonListener);
        buttonMessage.removeActionListener(buttonListener);
        checkWriteMessage.removeChangeListener(checkListener);
        outputPanel.clearAll();
        removeAll();
        buttonListener = null;
        checkListener = null;
    }

    // implements DeployStep
    public boolean isDataValid() {
        return DeployStepUtil.isDataValid(this);
    }

    // implements DeployStep
    public boolean isDataEqual(OtterProject project) {
        PathHandle projectPath = null;
        PathHandle panelPath = null;
        boolean    equal = true;

        projectRef = new WeakReference(project);
        if (project == null) {
            System.err.println("GeneralPanel.isDataEqual() - project null");
            return true;
        } else if (checkOverwrite.isSelected()
                   != project.isDeployOverwrite()) {
            equal = false;
        } else if (checkEcho.isSelected() != project.isDeployEcho()) {
            equal = false;
        } else if (checkDeployBuild.isSelected() != project.isDeployBuild()) {
            equal = false;
        }
        if (equal) {
            projectPath =
                PathHandle.createPathHandle(project.getOutputFilename());
            panelPath =
                PathHandle.createPathHandle(textOutputFilename.getText());
            if (!projectPath.equals(panelPath)) {
                equal = false;
            }
        }
        if (equal) {
            projectPath =
                PathHandle.createPathHandle(project.getDeployRootPath());
            panelPath = PathHandle.createPathHandle(textDeployRoot.getText());
            if (!projectPath.equals(panelPath)) {
                equal = false;
            }
        }
        return equal;
    }

    // implements DeployStep
    public void validateData() throws DataValidationException {
        PathHandle path = null;

        if (checkWriteMessage.isSelected()) {
            path = PathHandle.createPathHandle(textOutputFilename.getText());
            if (!isValidLogFile(path)) {
                throw new DataValidationException("Invalid message log: "
                                                  + path.getPath());
            }
        }
        path = PathHandle.createPathHandle(textDeployRoot.getText());
        if (!path.getPath().equals(textDeployRoot.getText())) {
            throw new DataValidationException("Invalid deploy root: "
                                              + textDeployRoot.getText());
        }
        if (!isValidDeployRoot(path)) {
            throw new DataValidationException("Invalid deploy root: "
                                              + textDeployRoot.getText());
        }
    }

    // implements DeployStep
    public boolean isBuilt(OtterProject project) {
        projectRef = new WeakReference(project);
        ran = DeployStepUtil.isBuilt(this, ran, project);
        return ran;
    }

    // implements DeployStep
    public boolean isSelectable(OtterProject project, DeployStep[] steps) {
        projectRef = new WeakReference(project);
        return true;
    }

    // implements DeployStep
    public void build(OtterProject project,
                      WriteListener listener) throws DataValidationException {
        projectRef = new WeakReference(project);
        if (!isBuilt(project)) {
            write(project);
            DeployBuilder.preBuild(this, project, listener);
            ran = true;
        }
    }

    // implements DeployStep
    public void read(OtterProject project) {
        PathHandle path = null;
        int        type = -1;

        projectRef = new WeakReference(project);
        checkOverwrite.setSelected(project.isDeployOverwrite());
        checkEcho.setSelected(project.isDeployEcho());
        checkDeployBuild.setSelected(project.isDeployBuild());
        type = project.getDeployType();
        if ((type > -1) && (type < comboType.getItemCount())) {
            comboType.setSelectedIndex(type);
        }
        path = PathHandle.createPathHandle(project.getOutputFilename());
        if (isValidLogFile(path)) {
            checkWriteMessage.setSelected(true);
            textOutputFilename.setText(path.getPath());
        } else {
            checkWriteMessage.setSelected(false);
            textOutputFilename.setText(new String());
        }
        textOutputFilename.setToolTipText(textOutputFilename.getText());
        path = PathHandle.createPathHandle(project.getDeployRootPath());
        if (isValidDeployRoot(path)) {
            textDeployRoot.setText(path.getPath());
        } else {
            textDeployRoot.setText(new String());
        }
        textDeployRoot.setToolTipText(textDeployRoot.getText());
    }

    // implements DeployStep
    public void write(OtterProject project) throws DataValidationException {
        projectRef = new WeakReference(project);
        validateData();
        if (!isDataEqual(project)) {
            project.setDeployOverwrite(checkOverwrite.isSelected());
            project.setDeployEcho(checkEcho.isSelected());
            if (checkWriteMessage.isSelected()) {
                project.setOutputFilename(PathHandle.createPathString(textOutputFilename.getText()));
            }
            project.setDeployRootPath(PathHandle.createPathString(textDeployRoot.getText()));
            project.setDeployBuild(checkDeployBuild.isSelected());

            // deploy type auto-updated
        }
    }

    //
    protected WriteListener getWriteListener() {
        return outputPanel;
    }

    protected Component[] getFirstFocusComponents() {
        Component[] comps = new Component[6];

        comps[0] = buttonDeployRoot;
        comps[1] = tabMessage;
        comps[2] = checkDeployBuild;
        comps[3] = checkWriteMessage;
        comps[4] = checkEcho;
        comps[5] = checkOverwrite;
        return comps;
    }

    //
    private boolean isValidDeployRoot(PathHandle ph) {
        boolean valid = false;

        if (!ph.isFile()) {
            if (ph.getParent() != null) {
                valid = ph.getParent().isDirectory();
            }
        }
        return valid;
    }

    private boolean isValidLogFile(PathHandle ph) {
        boolean valid = false;

        if (ph.hasExtension("txt")) {
            if (ph.isFile()) {
                valid = true;
            } else if (ph.getParent() != null) {
                valid = ph.getParent().isDirectory();
            }
        }
        return valid;
    }

    //
    private void chooseDeployRoot() {
        File       choice = null;
        PathHandle path = null;

        choice =
            SwingUtil.getDirectoryChoice(this, textDeployRoot.getText(),
                                         "Select deployment root directory");
        path = PathHandle.createPathHandle(choice);
        if (path.isDirectory()) {
            textDeployRoot.setText(path.getPath());
            textDeployRoot.setToolTipText(textDeployRoot.getText());
        }
    }

    private void chooseMessageFile() {
        File            choice = null;
        PathHandle      path = null;
        ExtensionFilter filter = null;

        filter = new ExtensionFilter();
        filter.setDescriptionTitle("Message Log");
        filter.addExtension("txt");
        choice =
            SwingUtil.getFileChoice(this, textOutputFilename.getText(),
                                    filter,
                                    "Select or enter a message log file");
        path = PathHandle.createPathHandle(choice);
        if (!path.isDirectory()) {
            if (!path.hasExtension("txt")) {
                path.setExtension("txt");
            }
            textOutputFilename.setText(path.getPath());
            textOutputFilename.setToolTipText(textOutputFilename.getText());
        }
    }

    private void checkWriteRefresh() {
        boolean write = checkWriteMessage.isSelected();

        buttonMessage.setEnabled(write);
        if (write) {
            textOutputFilename.setBackground(SystemColor.info);
        } else {
            textOutputFilename.setBackground(SystemColor.control);
        }
    }

    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonDeployRoot.addActionListener(buttonListener);
        buttonMessage.addActionListener(buttonListener);

        //
        checkListener = new LocalCheckListener();
        checkWriteMessage.addChangeListener(checkListener);
        outputPanel.setRows(4);

        //
        comboListener = new LocalComboListener();
        comboType.removeAllItems();
        comboType.addItem("Web Application");
//        comboType.addItem("Enhydra Service");
        comboType.addItem("Enhydra Application");
        comboType.setSelectedIndex(0);
        comboType.addItemListener(comboListener);
    }

    private void jbInit() throws Exception {
        panelDeployRoot =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutDeployRoot =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        textDeployRoot =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonDeployRoot =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        panelMessage = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  JPanel.class.getName());
        panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        checkWriteMessage =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkOverwrite =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        textOutputFilename =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonMessage =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        checkEcho = (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                                  JCheckBox.class.getName());
        checkDeployBuild =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        tabMessage =
            (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                            JTabbedPane.class.getName());
        panelMessageOption =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        outputPanel =
            (OutputPanel) Beans.instantiate(getClass().getClassLoader(),
                                            OutputPanel.class.getName());
        layoutMessageOption =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutMessage =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        panelType = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        comboType = (JComboBox) Beans.instantiate(getClass().getClassLoader(),
                                                  JComboBox.class.getName());
        layoutType =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        checkDeployBuild.setText("Deploy during project make/rebuild");
        textDeployRoot.setEnabled(false);
        textDeployRoot.setEditable(false);
        textDeployRoot.setText("DEPLOY ROOT");
        buttonDeployRoot.setText("...");
        checkWriteMessage.setText("Write to file");
        textOutputFilename.setEnabled(false);
        textOutputFilename.setEditable(false);
        textOutputFilename.setText("MESSAGE LOG");
        buttonMessage.setText("...");
        checkEcho.setText("Display during project make/rebuild");
        panelDeployRoot.setLayout(layoutDeployRoot);
        panelDeployRoot.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(),
                "Deploy root"));
        checkOverwrite.setText("Overwrite without warning");
        panelMessageOption.setLayout(layoutMessageOption);
        panelType.setLayout(layoutType);
        panelDeployRoot.add(textDeployRoot,
                            new GridBagConstraints(0, 0, 1, 1, 0.8, 0.0,
                                                   GridBagConstraints.WEST,
                                                   GridBagConstraints.HORIZONTAL,
                                                   new Insets(3, 5, 3, 5), 0,
                                                   0));
        panelDeployRoot.add(buttonDeployRoot,
                            new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                                   GridBagConstraints.CENTER,
                                                   GridBagConstraints.NONE,
                                                   new Insets(3, 5, 3, 5), 0,
                                                   0));
        panelMessage.setLayout(layoutMessage);
        panelMessage.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(),
                "Deploy messages"));
        panelMessageOption.add(checkWriteMessage,
                               new GridBagConstraints(0, 2, 2, 1, 0.1, 0.0,
                               GridBagConstraints.WEST,
                               GridBagConstraints.HORIZONTAL,
                               new Insets(0, 5, 2, 5), 0, 0));
        panelMessageOption.add(textOutputFilename,
                               new GridBagConstraints(0, 3, 1, 1, 0.9, 0.0,
                               GridBagConstraints.CENTER,
                               GridBagConstraints.HORIZONTAL,
                               new Insets(2, 5, 3, 5), 0, 0));
        panelMessageOption.add(buttonMessage,
                               new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0,
                               GridBagConstraints.CENTER,
                               GridBagConstraints.NONE,
                               new Insets(2, 5, 3, 5), 0, 0));
        panelMessageOption.add(checkEcho,
                               new GridBagConstraints(0, 1, 2, 1, 0.1, 0.0,
                               GridBagConstraints.CENTER,
                               GridBagConstraints.HORIZONTAL,
                               new Insets(3, 5, 3, 5), 0, 0));
        panelMessage.add(tabMessage, BorderLayout.CENTER);
        tabMessage.add(outputPanel, "Messages");
        tabMessage.add(panelMessageOption, "Options");
        panelType.setBorder((BorderFactory.createTitledBorder(new EtchedBorder(),
                "Deploy type")));
        panelType.add(comboType,
                      new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(2, 5, 2, 5), 0, 0));
        this.setLayout(layoutMain);
        this.add(panelFiller,
                 new GridBagConstraints(0, 6, 1, 1, 0.1, 0.01,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 0, 0, 0), 0, 0));
        this.add(checkDeployBuild,
                 new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 10, 2, 10), 0, 0));
        this.add(panelMessage,
                 new GridBagConstraints(0, 2, 2, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(panelDeployRoot,
                 new GridBagConstraints(0, 0, 2, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(checkOverwrite,
                 new GridBagConstraints(1, 3, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 10, 2, 10), 0, 0));
        this.add(panelType,
                 new GridBagConstraints(0, 1, 2, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 2, 0));
    }

    //
    private class LocalCheckListener implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            Object source = e.getSource();

            if (source == checkWriteMessage) {
                checkWriteRefresh();
            }
        }

    }

    //
    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonDeployRoot) {
                chooseDeployRoot();
            } else if (source == buttonMessage) {
                chooseMessageFile();
            }
        }

    }
    private void saveType() {
        OtterProject project = null;
        int          newType = OtterProject.TYPE_UNKNOWN;
        int          oldType = OtterProject.TYPE_UNKNOWN;

        if (projectRef != null) {
            project = (OtterProject) projectRef.get();
            newType = comboType.getSelectedIndex();
            oldType = project.getDeployType();
            if (newType != oldType) {
                project.setDeployType(newType);
                if (newType == OtterProject.TYPE_SERVICE) {
                    project.setDeployContent(false);
                    project.setDeployInput(false);
                } else if (oldType == OtterProject.TYPE_SERVICE) {
                    project.setDeployContent(true);
                    project.setDeployInput(true);
                }
                project.setDeployBootstrapPath(PathUtil.getAutoDeployPath(newType));
            }
        }
    }

    private class LocalComboListener implements ItemListener {
        public void itemStateChanged(ItemEvent event) {
            if (event.getStateChange() == ItemEvent.SELECTED) {
                saveType();
            }
        }

    }
}
