/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.map;

// Standard imports
import java.util.ArrayList;
import java.util.Arrays;

//
public class MapUtil {
    public static String[][] optimize(String[][] in) {
        String[][] out = new String[0][0];
        MapEntry[] entries = new MapEntry[0];

        entries = MapUtil.toEntryArray(in);
        entries = MapUtil.optimize(entries);
        out = MapUtil.toStringArray(entries);
        return out;
    }

    public static MapEntry[] optimize(MapEntry[] in) {
        MapEntry[] out = new MapEntry[0];

        out = reduce(in);
        out = sort(out);
        return out;
    }

    public static MapEntry[] combine(MapEntry[] setA, MapEntry[] setB) {
        ArrayList  listA = new ArrayList(Arrays.asList(setA));
        MapEntry[] both = new MapEntry[0];

        for (int i = 0; i < setB.length; i++) {
            if (!listA.contains(setB[i])) {
                listA.add(setB[i]);
            }
        }
        listA.trimToSize();
        both = new MapEntry[listA.size()];
        both = (MapEntry[]) listA.toArray(both);
        both = MapUtil.optimize(both);
        return both;
    }

    public static MapEntry[] toEntryArray(String[][] map) {
        MapEntry[] entries = new MapEntry[map.length];

        for (int i = 0; i < map.length; i++) {
            entries[i] = new MapEntry(map[i][0], map[i][1]);
        }
        return entries;
    }

    public static String[][] toStringArray(MapEntry[] entries) {
        String[][] map = new String[entries.length][2];

        for (int i = 0; i < entries.length; i++) {
            map[i][0] = entries[i].getFrom();
            map[i][1] = entries[i].getTo();
        }
        return map;
    }

    //
    //
    private static MapEntry[] sort(MapEntry[] in) {
        MapEntry[] out = new MapEntry[0];
        ArrayList  inList = null;
        ArrayList  outList = null;

        inList = new ArrayList(Arrays.asList(in));
        outList = new ArrayList();
        while (inList.size() > 0) {
            MapEntry max = (MapEntry) inList.get(0);

            for (int i = 0; i < inList.size(); i++) {
                MapEntry cursor = null;

                cursor = (MapEntry) inList.get(i);
                if (max.getFrom().length() < cursor.getFrom().length()) {
                    max = cursor;
                }
            }
            outList.add(max);
            inList.remove(max);
        }
        inList.trimToSize();
        outList.trimToSize();
        out = new MapEntry[outList.size()];
        out = (MapEntry[]) outList.toArray(out);
        outList.clear();
        return out;
    }

    private static MapEntry[] reduce(MapEntry[] in) {
        MapEntry[] out = new MapEntry[0];
        ArrayList  list = null;

        list = new ArrayList(Arrays.asList(in));
        for (int i = 0; i < in.length; i++) {
            MapEntry cursor = null;

            cursor = in[i];
            for (int j = 0; j < in.length; j++) {
                if (cursor.canReduce(in[j])) {
                    if (list.contains(in[j])) {
                        list.remove(in[j]);
                    }
                }
            }
        }
        list.trimToSize();
        out = new MapEntry[list.size()];
        out = (MapEntry[]) list.toArray(out);
        list.clear();
        return out;
    }

}
