/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte.node;

// ToolBox imports
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.PathHandle;

// Kelp imports
import org.enhydra.kelp.common.ResUtil;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterTemplateNode;

//netbeans imports:
import org.openide.util.NbBundle;
import org.openide.loaders.DataNode;
import org.openide.nodes.Node;


// Standard imports
import java.io.File;

/**
 *
 * @author  rees0234
 * @version
 */
public class ForteTemplateNode extends ForteNode implements OtterTemplateNode{
    /** Creates new ForteTemplateNode */
    public ForteTemplateNode(DeployDataObject n) throws ToolException {
        super(n);
        if (!PathUtil.isTemplate(this)) {
            StringBuffer buf = new StringBuffer();

            buf.append(NbBundle.getMessage (ForteTemplateNode.class, "MSG_Invalid_template_node"));

            buf.append('\n');
            buf.append(NbBundle.getMessage (ForteTemplateNode.class,"Node_file_path_0_"));
            buf.append(getFilePath());

            buf.append('\n');
            buf.append(NbBundle.getMessage (ForteTemplateNode.class,"Deploy_input_path_0_"));
            buf.append(getProject().getDeployInputPath());
            throw new ToolException(buf.toString());
        }
    }

    /**
     * Constructor declaration
     *
     *
     * @param otterNode
     * @param file
     */
    public ForteTemplateNode(OtterNode otterNode, String file) {
       super(otterNode, file);
    }

    public boolean isSelected()
    {
        String in = getProperty(NAME_SELECTED);
        return PropUtil.stringToBoolean(in, false);
    }

    public void setSelected(boolean b) {
        setProperty(NAME_SELECTED, PropUtil.booleanToString(b));
    }

    public String getProperty(String property)
    {
        String out = "";

        if (nativeNode != null) {
              out = ((DeployDataObject)nativeNode).getProperty(property);
        }

        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param value
     */
    public void setProperty(String property, String value)
    {
        if (nativeNode != null) {
            ((DeployDataObject)nativeNode).setProperty(property, value);
        }
    }

    // implements OtterTemplateNode
    public String getOutputPath() {
        return PathUtil.getOutputPath(this);
    }


}
