package org.enhydra.kelp.forte.services;

import java.io.File;
import java.io.IOException;
import java.util.Map;

import org.enhydra.kelp.forte.node.ForteProject;
import org.enhydra.kelp.common.deployer.RunParameters;
import org.enhydra.tool.ToolBoxInfo;

import org.openide.execution.*;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

/** Runs an external process to "execute" a type of file.
 *
 * @author root
 */
public class KelpExecutor extends ProcessExecutor {

    private static final NbProcessDescriptor DEFAULT = new NbProcessDescriptor (
                // PROCESS NAME:
                // This need not be the Java launcher, of course:
                "{" + Format.TAG_JAVAHOME + "}{" + Format.TAG_SEPARATOR + "}bin{" + Format.TAG_SEPARATOR + "}java",
               // You can modify the default format however you like, and insert or remove tags like so:
                "{" + MyFormat.TAG_MYOPTIONVAL + "} " //+
            );

    // The default value for an unconfigured executor.
    private String valueOfMyOption = "";

    public KelpExecutor () {
        // Override the default executor choice made in ProcessExecutor:
        setExternalExecutor (DEFAULT);
    }

    public String getRunOption () {
        if (valueOfMyOption.equals(""))
        {
            StringBuffer buf = new StringBuffer();
            ForteProject proj = new ForteProject();

            String easRoot = ToolBoxInfo.getEnhydraRoot();
            char sep = File.separatorChar;
            char psep = File.pathSeparatorChar;
            RunParameters rp = new RunParameters(proj);

            buf.append("-cp \"");
            buf.append(proj.getSourcePath());
            buf.append(psep);
            buf.append(easRoot);
            buf.append(sep);
            buf.append("lib");
            buf.append(sep);
            buf.append("build");
            buf.append(sep);
            buf.append("toolbox.jar");

            // dr 24.06.2002
            buf.append(psep);
            buf.append(easRoot);
            buf.append(sep);
            buf.append("lib");
            buf.append(sep);
            buf.append("tomcat.jar");

            String s = System.getProperty("java.home");
            s = s.substring(0, s.lastIndexOf("jre")-1);
            buf.append(psep);
            buf.append(s);
            buf.append(sep);
            buf.append("lib");
            buf.append(sep);
            buf.append("tools.jar");

            buf.append('\"');
            buf.append(' ');
            buf.append(rp.getTCcpArg());
            buf.append(' ');
            buf.append(rp.getPolicyArg());
            buf.append(' ');
            buf.append(rp.getBootArg());
            buf.append(" StartServer ");
            buf.append(rp.getAppParameters());
            setRunOption(buf.toString());
        }
        return valueOfMyOption;
    }

    public synchronized void setRunOption (String nue) {
        String old = valueOfMyOption;
        valueOfMyOption = nue;
        firePropertyChange ("valueOfMyOption", old, nue);
    }

    protected Process createProcess (ExecInfo info) throws IOException {
        return getExternalExecutor ().exec (new MyFormat (info,
                                            getClassPath (),
                                            getBootClassPath (),
                                            getRepositoryPath (),
                                            getLibraryPath (),
                                            getRunOption ()));
    }

    /*
    public HelpCtx getHelpCtx () {
        return new HelpCtx (KelpExecutor.class);
    }
    */

    static class MyFormat extends Format {

        // Define whatever other substitutable tags you want:
        static final String TAG_MYOPTIONVAL = "myOptionVal"; // this will appear as -myoption {myOptionVal}

        MyFormat (ExecInfo info, NbClassPath classPath, NbClassPath bootClassPath,
                  NbClassPath repositoryPath, NbClassPath libraryPath,
                  // Get the values to substitute from somewhere:
                  String valueOfMyOption) {
            // This fills in all the tags defined in Format, like TAG_LIBRARY, etc.:
            super (info, classPath, bootClassPath, repositoryPath, libraryPath);
            // Add definitions for your own tags here:
            Map map = getMap ();
            map.put (TAG_MYOPTIONVAL, valueOfMyOption);
        }

    }

}
