/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.jbuilder.wizard;

// ToolBox imports
import org.enhydra.tool.common.IconSet;
import org.enhydra.tool.common.ProgressMeter;

// JBuilder imports
import com.borland.jbuilder.node.HTMLFileNode;
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.node.Node;
import com.borland.primetime.wizard.BasicWizard;
import com.borland.primetime.wizard.BasicWizardPage;
import com.borland.primetime.wizard.Wizard;
import com.borland.primetime.wizard.WizardAction;
import com.borland.primetime.wizard.WizardHost;
import com.borland.primetime.wizard.WizardManager;
import com.borland.primetime.wizard.WizardPage;
import com.borland.primetime.util.VetoException;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.ResUtil;
import org.enhydra.kelp.common.ValidationException;
import org.enhydra.kelp.common.importer.ImportPaths;
import org.enhydra.kelp.common.importer.ImportTool;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.swing.KelpIconSet;
import org.enhydra.kelp.common.importer.ImportMapPanel;
import org.enhydra.kelp.common.importer.RootPanel;
import org.enhydra.kelp.common.importer.PathPanel;
import org.enhydra.kelp.common.importer.ImporterPanel;
import org.enhydra.kelp.jbuilder.wizard.AbstractBasicWizard;
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import org.enhydra.kelp.jbuilder.node.PrimeProject;

// Standard imports
import java.awt.BorderLayout;
import java.io.File;
import javax.swing.JPanel;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 * @author Paul Mahar
 */
public class ImportWizard extends AbstractBasicWizard {

    // string not to be resourced
    private final String       HELP_LINK = KelpInfo.getAddinHelpURL()
                                           + "#import";    // nores

    //
    private static String      WIZARD_NAME = res.getString("Enhydra_Import");
    private ImportPaths        paths = null;
    private OtterProject       project = null;
    public static WizardAction action = new LocalWizardAction();

    /**
     * Class declaration
     */
    static class LocalWizardAction extends WizardAction {
        static ResourceBundle res =
            ResourceBundle.getBundle("org.enhydra.kelp.jbuilder.Res");

        public LocalWizardAction() {
            super(ResUtil.format(res.getString("_0_dots"), ImportWizard.WIZARD_NAME),
                  'p', res.getString("importNotEmpty1"),
                  KelpIconSet.getSmallIcon(), KelpIconSet.getLargeIcon(),
                  false);
        }

        protected Wizard createWizard() {
            ImportWizard wizard = new ImportWizard();

            return wizard;
        }

        public void update(Object source) {
            setEnabled(AbstractBasicWizard.enableMenu);
        }

    }
    ;

    /**
     * Method declaration
     *
     *
     * @exception VetoException
     */
    public WizardPage finish(WizardPage currentPage,
                             WizardHost host) throws VetoException {
        ImportTool    tool = null;
        ProgressMeter progress = null;

        tool = new ImportTool();
        if (currentPage instanceof LocalWizardPage) {
            LocalWizardPage localPage = (LocalWizardPage) currentPage;

            localPage.checkPage();
        }
        progress = new ProgressMeter();
        progress.setOwner(Browser.getActiveBrowser());
        progress.setModal(true);
        progress.setTitle(getWizardTitle());
        tool.setPaths(paths);
        tool.addProgressListener(progress);
        progress.startDialogThread();
        tool.importFilesFromRoot();
        return super.finish(currentPage, host);
    }

    protected void finish() throws VetoException {
        super.finish();
    }

    /**
     * Method declaration
     *
     *
     * @param majorVersion
     * @param minorVersion
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
        WizardManager.registerWizardAction(action);
    }

    /**
     * Method declaration
     *
     *
     * @param currentPage
     * @param host
     */
    public String getHelpTopic() {
        return HELP_LINK;
    }

    /**
     * Method declaration
     *
     * @return
     */
    public String getDisplayName() {
        return ImportWizard.WIZARD_NAME;
    }

    public static String getName() {
        return ImportWizard.WIZARD_NAME;
    }

    public WizardPage invokeWizard(WizardHost host) {
        WizardPage page = null;
        boolean    invokeIt = false;

        try {
          KelpInfo.verifyIDEClassPath();
          invokeIt = true;
        } catch (Exception e) {
           JOptionPane.showMessageDialog(host.getDialogParent(),
                      e.getMessage(),
                      KelpInfo.getMessageBoxTitle(),
                      JOptionPane.ERROR_MESSAGE);
        }

        if (invokeIt) {

            setNativeProject((JBProject) host.getBrowser().getActiveProject());
            if (!isProjectEmpty(getNativeProject())) {
                StringBuffer buf = new StringBuffer();
                int          choice = -1;

                buf.append(res.getString("importNotEmpty2"));
                buf.append(res.getString("importNotEmpty3"));
                buf.append(res.getString("importNotEmpty4"));
                buf.append(res.getString("importNotEmpty5"));
                choice =
                    JOptionPane.showConfirmDialog(host.getDialogParent(),
                                                  buf.toString(),
                                                  ResUtil.format(res.getString("_0_Wizard_Error"), getDisplayName()),
                                                  JOptionPane.WARNING_MESSAGE);
                if (choice == JOptionPane.CANCEL_OPTION) {
                    invokeIt = false;
                }
            }
            if (invokeIt) {
                project = new PrimeProject(getNativeProject());
                paths = new ImportPaths();
                paths.setProject(project);
                String rootPath = project.getRootPath();

                if (rootPath != null && rootPath.trim().length() > 0) {
                    try {
                        paths.setRootPath(rootPath);
                    } catch (ValidationException e) {
                        e.printStackTrace();
                    }
                }
                LocalWizardPage rootPage = new LocalWizardPage();
                LocalWizardPage pathPage = new LocalWizardPage();
                LocalWizardPage mapPage = new LocalWizardPage();

                rootPage.setPanel(new RootPanel());
                rootPage.setPaths(paths);
                pathPage.setPanel(new PathPanel());
                pathPage.setPaths(paths);
                mapPage.setPanel(new ImportMapPanel());
                mapPage.setPaths(paths);
                addWizardPage(rootPage);
                addWizardPage(pathPage);
                addWizardPage(mapPage);
                page = super.invokeWizard(host);
            }
        }
        return page;
    }

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     */
    private boolean isProjectEmpty(JBProject nativeProject) {
        Node[]  nodes = null;
        boolean empty = true;

        if (nativeProject != null) {
            nodes = nativeProject.getChildren();
        }
        if (nodes != null) {
            if (nodes.length > 1) {
                empty = false;
            } else if (nodes.length == 1) {
                if (!(nodes[0] instanceof HTMLFileNode)) {
                    empty = false;
                }
            }
        }
        return empty;
    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     */
    private class LocalWizardPage extends BasicWizardPage {
        private ImporterPanel panel;

        /**
         * Constructor declaration
         *
         */
        public LocalWizardPage() {
            BorderLayout layout = new BorderLayout();

            layout.setHgap(5);
            layout.setVgap(5);
            this.setLayout(layout);
            if (KelpInfo.isToolBoxVersionInSynch()) {
                this.setLargeIcon(IconSet.getOtterWizard());
            }
        }

        /**
         * Method declaration
         *
         * @param host
         */
        public void activated(WizardHost host) {
            panel.activated();
        }

        /**
         * Method declaration
         *
         * @exception VetoException
         */
        public void checkPage() throws VetoException {
            try {
                panel.checkPage();
            } catch (ValidationException e) {
                JOptionPane.showMessageDialog(getWizardHost().getDialogParent(),
                                              e.getMessage(),
                                              getDisplayName(),
                                              JOptionPane.ERROR_MESSAGE);
                throw new VetoException(e.toString());
            }
        }

        /**
         * Method declaration
         *
         * @param importer
         */
        public void setPaths(ImportPaths paths) {
            panel.setPaths(paths);
        }

        /**
         * Method declaration
         *
         * @return
         */
        public ImportPaths getPaths() {
            return panel.getPaths();
        }

        protected void setPanel(ImporterPanel panel) {
            this.panel = panel;
            this.setPageTitle(panel.getPageTitle());
            this.setInstructions(panel.getInstructions());
            this.add(panel, BorderLayout.CENTER);
        }

        protected JPanel getPanel() {
            return panel;
        }

    }
}
