/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.node;

// ToolBox imports
import org.enhydra.tool.common.PathHandle;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.common.node.PropertyKeys;

// JBuilder Foundation imports
import com.borland.primetime.node.FileNode;
import com.borland.primetime.vfs.InvalidUrlException;

// Standard imports
import java.io.IOException;

//
/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class PrimeDocumentNode extends PrimeNode
    implements OtterDocumentNode {

    //
    public PrimeDocumentNode(FileNode n) {
        super(n);
    }

    /**
     * Constructor declaration
     *
     *
     * @param otterNode
     * @param path
     */
    public PrimeDocumentNode(OtterNode otterNode, String path) {
        super(otterNode, path);
    }

    public void setSelected(boolean b) {
        super.setSelected(b);
        if (b) {
            setStatic(false);
        }
    }

    public boolean isStatic() {
        String in = getProperty(PropertyKeys.NAME_RESOURCE_STATIC);

        return PropUtil.stringToBoolean(in, false);
    }

    public void setStatic(boolean b) {
        setProperty(PropertyKeys.NAME_RESOURCE_STATIC,
                    PropUtil.booleanToString(b));
        if (b) {
            setSelected(false);
        }
    }

    public void preCompile() {
        if (getNativeNode() instanceof FileNode) {
            FileNode nativeNode = (FileNode) getNativeNode();

            if (nativeNode.isModified()) {
                try {
                    nativeNode.save();
                } catch (InvalidUrlException e) {
                    e.printStackTrace();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

}
