/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jdeveloper.node;

// AddinCore
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.PropertyKeys;

// JDeveloper imports
import borland.jbuilder.addin.Container;
import borland.jbuilder.addin.Node;
import borland.jbuilder.addin.Url;

// Standard imports
import java.io.File;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class OracleNode implements OtterFileNode {
    private Node      nativeNode = null;
    private Throwable exception = null;

    /**
     * Constructor declaration
     *
     */
    public OracleNode() {}

    /**
     * Constructor declaration
     *
     *
     * @param n
     */
    public OracleNode(Node n) {
        nativeNode = n;
    }

    /**
     * Constructor declaration
     *
     * @param otterNode
     * @param path
     */
    public OracleNode(OtterNode otterNode, String path) {
        Node      newNative = null;
        Container container = null;
        Url       url = null;

        if (otterNode.getNativeNode() instanceof Container) {
            container = (Container) otterNode.getNativeNode();
        } else {
            container = (Container) otterNode.getProject().getNativeProject();
        }
        url = new Url(path);
        newNative = container.findNode(url);
        if (newNative == null) {
            newNative = container.createNode(url, true, false);
            container.addNode(newNative, false);
            container.save(true);
        }
        setNativeNode(newNative);
    }

    /**
     * Method declaration
     *
     *
     * @param n
     */
    public void setNativeNode(Object n) {
        nativeNode = (Node) n;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public Object getNativeNode() {
        return nativeNode;
    }

    public OtterProject getProject() {
        return (new OracleProject(nativeNode.getProject()));
    }

    public OtterNode getParent() {
        return getProject();
    }

    public String getXMLCOptionFilePath() {
        return getProperty(PropertyKeys.NAME_XMLC_OPTION_FILEPATH);
    }

    public void setXMLCOptionFilePath(String n) {
        setProperty(PropertyKeys.NAME_XMLC_OPTION_FILEPATH, n);
    }

    public String getXMLCParameters() {
        String out = new String();

        out = getProperty(PropertyKeys.NAME_XMLC_PARAMETERS);
        out = PropUtil.cleanXMLCParameters(out);
        return out;
    }

    public void setXMLCParameters(String in) {
        String out = new String();

        out = PropUtil.cleanXMLCParameters(in);
        setProperty(PropertyKeys.NAME_XMLC_PARAMETERS, out);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getFilePath() {
        String out = null;

        out = nativeNode.getFilename();
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     *
     * @return
     */
    public String getProperty(String property) {
        String out = null;
        String path = new String();
        int    hash = -1;

        path = getFilePath();
        path = PathUtil.compressPathRelativeToProject(this, path);
        hash = path.hashCode();
        out = nativeNode.getProject().getProperty(property + '[' + hash
                                                  + ']');
        return out;
    }

    /**
     * Set a property through the project using a hashcode of
     * the filename. The setProperty on the node is not
     * implemented in JDeveloper.
     *
     * @param property
     * @param value
     */
    public void setProperty(String property, String value) {
        String path = new String();
        int    hash = -1;

        path = getFilePath();
        path = PathUtil.compressPathRelativeToProject(this, path);
        hash = path.hashCode();
        nativeNode.getProject().setProperty(property + '[' + hash + ']',
                                            value);
    }

    public void setProperty(String property, int i) {
        String value = Integer.toString(i);

        setProperty(property, value);
    }

    public boolean isSelected() {
        String in = getProperty(PropertyKeys.NAME_SELECTED);

        return PropUtil.stringToBoolean(in, false);
    }

    public void setSelected(boolean b) {
        setProperty(PropertyKeys.NAME_SELECTED, PropUtil.booleanToString(b));
    }

    /**
     * Save Exception information for reporting. This lets exceptions from
     * multiple nodes to be reported in a batch.
     *
     * @param e
     * An exception that occured while invoking XMLC.
     */
    public void setException(Throwable e) {
        exception = e;
    }

    /**
     * Get an exception that occured when invoking XMLC. Null if the node
     * has not been compiled or was compiled without error.
     */
    public Throwable getException() {
        return exception;
    }

    public void save() {
        nativeNode.save(true);
    }

}
