/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jdeveloper.node;

// JDeveloper imports
import borland.jbuilder.addin.Node;

// Kelp imports
import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterImageFileNode;
import org.enhydra.kelp.common.node.OtterJavaFileNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;

// Standard imports
import java.io.File;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class OracleNodeFactory extends OtterNodeFactory {

    /**
     * Method declaration
     *
     *
     * @param otterNode
     * @param name
     *
     * @return
     */
    public OtterFolderNode createFolderNode(OtterNode otterNode,
                                            String name) {
        OtterFolderNode folder = null;

        folder = new OracleFolderNode(otterNode, name);
        return folder;
    }

    /**
     * Method declaration
     *
     *
     * @param otterNode
     * @param file
     *
     * @return
     */
    public OtterJavaFileNode createJavaFileNode(OtterNode otterNode,
                                                String file) {
        OtterJavaFileNode newNode = null;

        newNode = new OracleJavaFileNode(otterNode,
                                         file.replace('/',
                                         File.separatorChar));
        return newNode;
    }

    /**
     * Method declaration
     *
     *
     * @param otterNode
     * @param file
     *
     * @return
     */
    public OtterDocumentNode createDocumentNode(OtterNode otterNode,
                                                String file) {
        OtterDocumentNode newNode = null;

        newNode = new OracleDocumentNode(otterNode,
                                         file.replace('/',
                                         File.separatorChar));
        return newNode;
    }

    /**
     * Method declaration
     *
     *
     * @param otterNode
     * @param file
     *
     * @return
     */
    public OtterImageFileNode createImageFileNode(OtterNode otterNode,
            String file) {
        OtterImageFileNode newNode = null;

        newNode = new OracleImageFileNode(otterNode,
                                          file.replace('/',
                                          File.separatorChar));
        return newNode;
    }

    /**
     * Method declaration
     *
     *
     * @param otterNode
     * @param file
     *
     * @return
     */
    public OtterTextFileNode createTextFileNode(OtterNode otterNode,
                                                String file) {
        OtterTextFileNode newNode = null;

        newNode = new OracleTextFileNode(otterNode,
                                         file.replace('/',
                                         File.separatorChar));
        return newNode;
    }

    public OtterTemplateNode createTemplateNode(OtterNode otterNode,
                                                String file) {
        OtterTemplateNode newNode = null;

        newNode = new OracleTemplateNode(otterNode,
                                         file.replace('/',
                                         File.separatorChar));
        return newNode;
    }

    public OtterTemplateNode getTemplateNode(OtterNode otterNode) {
        OtterTemplateNode newNode = null;
        Node nativeNode = null;
        nativeNode = (Node) otterNode.getNativeNode();
        newNode = new OracleTemplateNode(nativeNode);
        return newNode;
    }


    public OtterJavaFileNode[] replaceGeneratedSource(OtterProject project,
            OtterNode node, String[] files) {

        // nothing extra for Win32.
        OtterJavaFileNode[] nodes = new OtterJavaFileNode[files.length];

        for (int i = 0; i < files.length; i++) {
            nodes[i] = createJavaFileNode(node,
                                          files[i].replace('/',
                                          File.separatorChar));
        }
        return nodes;
    }

}
