/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the Licenseb for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 * Travis Shook
 *
 */
package org.enhydra.kelp.jdeveloper.tool;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.IconSet;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.ExtensionFilter;

// JDeveloper imports
import borland.jbuilder.addin.Context;
import borland.jbuilder.addin.IdeEnvironment;
import borland.jbuilder.addin.MenuCategory;
import borland.jbuilder.addin.MenuItem;
import borland.jbuilder.addin.MenuItemAdaptor;
import borland.jbuilder.addin.MenuListener;
import borland.jbuilder.addin.MenuLocations;
import borland.jbuilder.addin.MenuManager;
import borland.jbuilder.addin.Project;
import borland.jbuilder.addin.VersionedAddin;

// AddinCore
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ResUtil;

// AddinJDeveloper
import org.enhydra.kelp.jdeveloper.HelpBrowser;

// JDK
import java.awt.event.ActionEvent;
import java.io.Serializable;
import java.util.ResourceBundle;

//
abstract public class AbstractBasicTool implements MenuListener,
        Serializable, VersionedAddin {

    //
    static int            count = 0;
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.jdeveloper.Res");    // nores

    // string not to be resourced
    private final String  VERSION = "2.0";    // nores

    //
    protected Project     nativeProject = null;

    abstract protected String getTitle();
    abstract protected void invoke();

    // implements borland.jbuilder.addin.MenuListener
    public void menuActivated(MenuCategory menuCategory, Context context) {}

    public void contextMenuCreation(MenuCategory menuCategory,
                                    Context context) {}

    public void startup(IdeEnvironment ide) {
        MenuManager  manager = null;
        MenuCategory category = null;
        MenuItem     item = null;

        manager = ide.getMenuManager();
        category =
            manager.getMenuCategory(MenuLocations.menuNames[MenuLocations.ToolsMenu]);
        item = manager.createMenuItem(getTitle(), getTitle(),
                                      new LocalItemListener());
        manager.insertMenuItem(item, category,
                               MenuLocations.BeforeIDEOptions);
        count++;
        if (count == 2) {
            item = manager.createMenuItem("-", "enhydra-seperator", null);
            manager.insertMenuItem(item, category,
                                   MenuLocations.BeforeIDEOptions);
        }
    }

    /**
     * For the VersionAddin Interface.
     */
    public String version() {
        return VERSION;
    }

    /**
     * Method declaration
     *
     *
     * @param projectClassPath
     *
     * @return
     */
    public void verifyAllClassPaths() throws Exception {
        boolean      valid = false;
        boolean      isEnhydra3 = ToolBoxInfo.isEnhydra3();
        StringBuffer message = new StringBuffer();
        String       projectClassPath = new String();

        KelpInfo.verifyIDEClassPath();
        if (nativeProject != null) {
            projectClassPath = nativeProject.getClassPath();
            if (isEnhydra3) {
                valid =
                    projectClassPath.toLowerCase().indexOf(Constants.FILE_ENHYDRA_JAR)
                    > 0;
            } else {
                valid =
                    projectClassPath.toLowerCase().indexOf(Constants.FILE_XMLC_JAR)
                    > 0;
            }
            if (!valid) {
                if (isEnhydra3) {
                    message.append(res.getString("EnhydraLibError1"));
                } else {
                    message.append(res.getString("EnhydraLibError2"));
                }
                message.append('\n');
                message.append(res.getString("EnhydraLibError3"));
                throw new Exception(message.toString());
            }
        }
    }

    public void browseKelpHelp(String anchor) {
        HelpBrowser browser = new HelpBrowser();
        browser.open(anchor);
    }

    private class LocalItemListener extends MenuItemAdaptor {
        public void actionPerformed(ActionEvent e) {
            invoke();
        }

        public void itemValidate(MenuItem item, MenuCategory cat,
                                 Context con) {
            item.setEnabled(con.getNode() != null);
        }

        public boolean actionPending(ActionEvent e) {
            return false;
        }

    }
}
