/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.multiServer.launch;

// Standard imports
import java.awt.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

//
public class InfoPanel extends JPanel {
    private GridBagLayout layoutMain;
    private JPanel        panelUser;
    private JPanel        panelSystem;
    private JPanel        panelJava;
    private JPanel        panelClassPath;
    private GridBagLayout layoutUser;
    private GridBagLayout layoutSystem;
    private GridBagLayout layoutJava;
    private JLabel        labelName;
    private JLabel        labelWorking;
    private JLabel        labelHome;
    private JTextField    textName;
    private JTextField    textWorking;
    private JTextField    textHome;
    private JLabel        labelSystemName;
    private JLabel        labelSystemArch;
    private JTextField    textSystemName;
    private JTextField    textSystemArch;
    private JLabel        labelJavaHome;
    private JLabel        labelVendor;
    private JLabel        labelVersion;
    private JTextField    textJavaHome;
    private JTextField    textVendor;
    private JTextField    textVersion;
    private JTextArea     textClassPath;
    private JScrollPane   scrollClassPath;
    private BorderLayout  layoutClassPath;

    public InfoPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private void pmInit() {
        textHome.setText(System.getProperty("user.home"));
        textWorking.setText(System.getProperty("user.dir"));
        textName.setText(System.getProperty("user.name"));
        textSystemName.setText(System.getProperty("os.name"));
        textSystemArch.setText(System.getProperty("os.arch"));
        textJavaHome.setText(System.getProperty("java.home"));
        textVendor.setText(System.getProperty("java.vendor"));
        textVersion.setText(System.getProperty("java.version"));
        textClassPath.append(System.getProperty("java.class.path"));
    }

    private void jbInit() throws Exception {
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelUser = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        panelSystem = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        panelJava = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        panelClassPath =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutUser =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutSystem =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutJava =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        labelName = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                               JLabel.class.getName());
        labelWorking = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                  JLabel.class.getName());
        labelHome = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                               JLabel.class.getName());
        textName = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        textWorking =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        textHome = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        labelSystemName =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        labelSystemArch =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        textSystemName =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        textSystemArch =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        labelJavaHome =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        labelVendor = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                 JLabel.class.getName());
        labelVersion = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                  JLabel.class.getName());
        textJavaHome =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        textVendor =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        textVersion =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        textClassPath =
            (JTextArea) Beans.instantiate(getClass().getClassLoader(),
                                          JTextArea.class.getName());

        // User
        layoutClassPath =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        labelName.setText("Name:");
        labelWorking.setText("Working directory:");
        labelHome.setText("Home directory:");
        textName.setEditable(false);
        textName.setText("USER HOME");
        textWorking.setEditable(false);
        textWorking.setText("USER WORKING DIRECTORY");
        textHome.setEditable(false);
        textHome.setText("USER HOME DIRECTORY");
        panelUser.setLayout(layoutUser);
        panelUser.setBorder(BorderFactory.createTitledBorder("User"));
        labelJavaHome.setText("Java home:");
        labelVendor.setText("Vendor:");
        labelVersion.setText("Version:");
        textJavaHome.setEditable(false);
        textJavaHome.setText("JAVA HOME");
        textVendor.setEditable(false);
        textVendor.setText("JAVA VENDOR");
        textVersion.setEditable(false);
        textVersion.setText("JAVA VERSION");
        textClassPath.setLineWrap(true);
        textClassPath.setWrapStyleWord(true);
        panelUser.add(labelName,
                      new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(5, 5, 2, 5), 0, 0));
        panelUser.add(labelWorking,
                      new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(2, 5, 2, 5), 0, 0));
        panelUser.add(labelHome,
                      new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(2, 5, 5, 5), 0, 0));
        panelUser.add(textName,
                      new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(5, 5, 2, 5), 100, 0));
        panelUser.add(textWorking,
                      new GridBagConstraints(1, 1, 1, 1, 0.1, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(2, 5, 2, 5), 100, 0));
        panelUser.add(textHome,
                      new GridBagConstraints(1, 2, 1, 1, 0.1, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(2, 5, 5, 5), 0, 0));

        // System
        labelSystemName.setText("Operating system:");
        labelSystemArch.setText("Architecture:");
        textSystemName.setEditable(false);
        textSystemName.setText("OPERATING SYSTEM");
        textSystemArch.setEditable(false);
        textSystemArch.setText("ARCHITECHTURE");
        panelSystem.setLayout(layoutSystem);
        panelSystem.setBorder(BorderFactory.createTitledBorder("System"));
        panelSystem.add(labelSystemName,
                        new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                               GridBagConstraints.WEST,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(5, 5, 2, 5), 0, 0));
        panelSystem.add(labelSystemArch,
                        new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                                               GridBagConstraints.WEST,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(2, 5, 5, 5), 0, 0));
        panelSystem.add(textSystemName,
                        new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.WEST,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(5, 5, 2, 5), 100,
                                               0));
        panelSystem.add(textSystemArch,
                        new GridBagConstraints(1, 1, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.WEST,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(2, 5, 5, 5), 100,
                                               0));

        // Java
        panelJava.setLayout(layoutJava);
        panelJava.setBorder(BorderFactory.createTitledBorder("Java"));
        panelJava.add(labelJavaHome,
                      new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(5, 5, 2, 5), 40, 0));
        panelJava.add(labelVendor,
                      new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(2, 5, 2, 5), 0, 0));
        panelJava.add(labelVersion,
                      new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(2, 5, 5, 5), 0, 0));
        panelJava.add(textJavaHome,
                      new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(5, 5, 2, 5), 100, 0));
        panelJava.add(textVendor,
                      new GridBagConstraints(1, 1, 1, 1, 0.1, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(2, 5, 2, 5), 100, 0));
        panelJava.add(textVersion,
                      new GridBagConstraints(1, 2, 1, 1, 0.1, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(2, 5, 5, 5), 100, 0));

        // CLASSPATH
        panelClassPath.setBorder(BorderFactory.createTitledBorder("Class Path"));
        panelClassPath.setLayout(layoutClassPath);
        textClassPath.setEditable(false);
        textClassPath.setRows(5);
        scrollClassPath = new JScrollPane(textClassPath);
        panelClassPath.add(scrollClassPath, BorderLayout.CENTER);

        // MAIN
        this.setMaximumSize(new Dimension(600, 440));
        this.setMinimumSize(new Dimension(600, 440));
        this.setPreferredSize(new Dimension(600, 440));
        this.setLayout(layoutMain);
        this.add(panelUser,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 2, 5), 0, 0));
        this.add(panelSystem,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(panelJava,
                 new GridBagConstraints(0, 2, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(panelClassPath,
                 new GridBagConstraints(0, 3, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(2, 5, 5, 5), 10, 60));
    }

}
