/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.multiServer.launch;

// Kernel imports
import org.enhydra.multiServer.kernel.Kernel;

// Standard imports
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.util.Date;
import javax.swing.border.*;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;

//
public class LaunchPanel extends JPanel {
    private GridBagLayout layoutMain;
    private JToolBar      toolBar;
    private JButton       buttonStart;
    private JButton       buttonStop;
    private JButton       buttonAdmin;
    private ConfPanel     confPanel;
    private OutputPanel   outputPanel;
    private JPanel        panelStatus;
    private JLabel        labelStatus;
    private JTabbedPane   tabPane;
    private GridBagLayout layoutStatus;

    public LaunchPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    protected void initPaths() {
        confPanel.initPaths();
    }

    protected void refreshStatus(String status, String message,
                                 boolean starting) {
        Date   nowDate = new Date(System.currentTimeMillis());
        String now = DateFormat.getDateTimeInstance().format(nowDate);

        if (getApp() != null) {
            labelStatus.setText(status + " at " + now);
            if ((message != null) && (message.trim().length() > 0)
                    && (outputPanel != null)) {
                outputPanel.appendMessage(message);
                outputPanel.appendMessage("\n");
            }
            if (starting) {
                tabPane.setSelectedIndex(0);
                buttonStart.setEnabled(false);
                buttonStop.setEnabled(true);
                buttonAdmin.setEnabled(false);
            } else if (Kernel.getKernel() == null) {
                buttonStart.setEnabled(true);
                buttonStop.setEnabled(false);
                buttonAdmin.setEnabled(false);
            } else if (getApp().isServerRunning()) {
                buttonStart.setEnabled(false);
                buttonStop.setEnabled(true);
                buttonAdmin.setEnabled(true);
            } else {
                buttonStart.setEnabled(true);
                buttonStop.setEnabled(true);
                buttonAdmin.setEnabled(false);
            }
        }
    }

    protected void startServer() {
        getApp().startServer();
    }

    protected void stopServer() {
        int choice =
            JOptionPane.showConfirmDialog(this,
                                          "Shutting down the server will also close the \n"
                                          + "launcher application. Continue with shutdown?",
                                          "Enhydra Launcher",
                                          JOptionPane.OK_CANCEL_OPTION,
                                          JOptionPane.QUESTION_MESSAGE);

        if (choice == JOptionPane.OK_OPTION) {
            getApp().stopServer();
        }
    }

    protected void gotoAdmin() {
        if (!getApp().isServerRunning()) {
            int result =
                JOptionPane.showConfirmDialog(getTopLevelAncestor(),
                                              "The Enhydra server is required to use the Admin servlet \n"
                                              + "and it may not have been started. Do you want to continue?",
                                              "Go to Admin Servlet",
                                              JOptionPane.YES_NO_OPTION);

            if (result == JOptionPane.NO_OPTION) {
                return;
            }
        }
        if (getApp().getBrowser() == null) {
            confPanel.selectBrowser();
            getApp().storeAppProperties();
        }
        if (getApp().getBrowser() != null) {
            StringBuffer cmd = new StringBuffer();
            cmd.append(getApp().getBrowser());
            cmd.append(" ");
            cmd.append("http://localhost:8001");
            try {
                Runtime.getRuntime().exec(cmd.toString());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    protected void viewMessages() {
        tabPane.setSelectedIndex(0);
    }

    protected void viewSettings() {
        tabPane.setSelectedIndex(1);
        confPanel.firstFocus();
    }

    protected boolean isMessageTabSelected() {
        return (tabPane.getSelectedIndex() == 0);
    }

    private void pmInit() {
        Border borderText = null;
        Border borderStatus = null;

        borderText = BorderFactory.createBevelBorder(1);
        borderStatus = BorderFactory.createLoweredBevelBorder();
        panelStatus.setBorder(borderStatus);
        ImageIcon start =
            new ImageIcon(ClassLoader.getSystemResource("org/enhydra/multiServer/launch/media/start.gif"));
        ImageIcon stop =
            new ImageIcon(ClassLoader.getSystemResource("org/enhydra/multiServer/launch/media/stop.gif"));
        ImageIcon admin =
            new ImageIcon(ClassLoader.getSystemResource("org/enhydra/multiServer/launch/media/admin.gif"));

        buttonStart.setIcon(start);
        buttonStart.setText(new String());
        buttonStop.setIcon(stop);
        buttonStop.setText(new String());
        buttonAdmin.setIcon(admin);
        buttonAdmin.setText(new String());
        labelStatus.setText(new String());
        outputPanel.initCapture();
    }

    private void jbInit() throws Exception {
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        toolBar = (JToolBar) Beans.instantiate(getClass().getClassLoader(),
                                               JToolBar.class.getName());
        buttonStart = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                  JButton.class.getName());
        buttonStop = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        buttonAdmin = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                  JButton.class.getName());
        confPanel = (ConfPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  ConfPanel.class.getName());
        outputPanel =
            (OutputPanel) Beans.instantiate(getClass().getClassLoader(),
                                            OutputPanel.class.getName());
        panelStatus = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        labelStatus = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                 JLabel.class.getName());
        tabPane =
            (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                            JTabbedPane.class.getName());
        layoutStatus =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonStart.setRequestFocusEnabled(false);
        buttonStart.setToolTipText("Start Server");
        buttonStart.setBorderPainted(false);
        buttonStart.setFocusPainted(false);
        buttonStart.setText("->");
        buttonStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                startServer();
            }

        });
        buttonStop.setEnabled(false);
        buttonStop.setRequestFocusEnabled(false);
        buttonStop.setToolTipText("Shutdown Server");
        buttonStop.setBorderPainted(false);
        buttonStop.setFocusPainted(false);
        buttonStop.setText("X");
        buttonStop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                stopServer();
            }

        });
        buttonAdmin.setEnabled(false);
        buttonAdmin.setRequestFocusEnabled(false);
        buttonAdmin.setToolTipText("Go to Admin Servlet");
        buttonAdmin.setBorderPainted(false);
        buttonAdmin.setFocusPainted(false);
        buttonAdmin.setText("A");
        buttonAdmin.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                gotoAdmin();
            }

        });

        // toolbar
        toolBar.setFloatable(false);
        toolBar.add(buttonStart, null);
        toolBar.add(buttonStop, null);
        toolBar.add(buttonAdmin, null);

        // tab
        tabPane.addTab("Messages", outputPanel);
        tabPane.addTab("Settings", confPanel);
        labelStatus.setText("Status");
        panelStatus.setLayout(layoutStatus);
        panelStatus.add(labelStatus,
                        new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(1, 5, 1, 5), 100,
                                               22));
        this.setLayout(layoutMain);
        this.add(toolBar,
                 new GridBagConstraints(0, 0, GridBagConstraints.REMAINDER,
                                        1, 0.1, 0.1,
                                        GridBagConstraints.NORTH,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 5, 5), 0, 0));
        this.add(tabPane,
                 new GridBagConstraints(0, 0, 1, 1, 0.5, 0.5,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(50, 4, 5, 4), 300, 50));
        this.add(panelStatus,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.SOUTH,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 0, 0));
    }

    private LaunchApp getApp() {
        return LaunchApp.app;
    }

}
