/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.multiServer.launch;

// Kernel imports
import org.enhydra.multiServer.launch.capture.Printer;
import org.enhydra.multiServer.launch.capture.PrintEvent;
import org.enhydra.multiServer.launch.capture.PrintListener;

// Standard imports
import java.io.IOException;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.awt.*;
import javax.swing.*;
import java.beans.Beans;
public class OutputPanel extends JPanel implements PrintListener {
    private BorderLayout layoutMain;
    private JScrollPane  scrollOutput;
    private JTextArea    textOutput;
    private boolean      critical = false;
    private Printer      errPrinter = new Printer(PrintEvent.ERROR);
    private Printer      outPrinter = new Printer(PrintEvent.OUTPUT);

    public OutputPanel() {
        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void onPrint(PrintEvent event) {
        final String output = event.getString();

        appendMessage(output);
    }

    protected void initCapture() {
        Printer.addPrintListener(this);
        captureOutput();
    }

    private void captureOutput() {
        if (!LaunchApp.debug) {
            System.setErr(errPrinter);
            System.setOut(outPrinter);
        }
    }

    private LaunchApp getApp() {
        return LaunchApp.app;
    }

    protected void appendMessage(final String out) {
        String message = new String(out);

        appendToFile(message);
        if (!critical) {
            if ((message != null)
                    && (message.indexOf("kernel,CRITICAL") > -1)) {
                if (message.length() > 100) {
                    message = message.substring(0, 75) + "...";
                }
                message = message + "\n\n See " + getApp().getLogFilePath()
                          + "\n for more information.";
                getApp().runKiller(30);
                critical = true;
                JOptionPane.showMessageDialog(this, message,
                                              "Critical Error: Server will shutdown in 30 seconds",
                                              JOptionPane.ERROR_MESSAGE);
            }
        }
        Runnable sendOutput = new Runnable() {
            public void run() {
                textOutput.append(out + "\n");
            }

        };

        try {
            SwingUtilities.invokeLater(sendOutput);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void appendToFile(String message) {
        if (getApp() != null) {
            try {
                PrintWriter logWriter = getApp().getLogWriter();

                logWriter.println(message);
            } catch (IOException e) {
                getApp().freeOutput();
                e.printStackTrace();
                captureOutput();
            }
        }
    }

    private void jbInit() throws Exception {
        layoutMain =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        textOutput =
            (JTextArea) Beans.instantiate(getClass().getClassLoader(),
                                          JTextArea.class.getName());
        textOutput.setEditable(false);
        textOutput.setRows(15);
        scrollOutput = new JScrollPane(textOutput);
        this.setLayout(layoutMain);
        this.add(scrollOutput, BorderLayout.CENTER);
    }

}
