/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive;

// Build
//import org.enhydra.ant.taskdefs.EJBClient;

// ToolBox
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.ResUtil;
import org.enhydra.tool.archive.Descriptor;
import org.enhydra.tool.archive.xml.EarDescriptorHandler;

// JDK
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.ResourceBundle;
import java.util.jar.Attributes;
import java.util.jar.JarFile;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;
import java.util.jar.JarOutputStream;
import java.util.jar.Manifest;

//
public class EarBuilder extends JarBuilder {

    // not to be resourced
    private final String  APPLICATION_XML =
        "META-INF/application.xml";    // nores
    private InputStream[] mergeDescriptors = new InputStream[0];
    private PathHandle    tmpDir = null;

    //
    //
    public EarBuilder() {}

    public EarPlan getEarPlan() {
        EarPlan ep = null;

        if (getPlan() instanceof EarPlan) {
            ep = (EarPlan) getPlan();
        }
        return ep;
    }

    public File buildArchive() throws ArchiveException {
        File         jar = null;
        String       root = null;
        File[]       files = new File[0];
        Descriptor[] dd = new Descriptor[0];
        boolean      userAppDD = false;

        root = getPlan().getClassRoot();
        dd = getPlan().getDescriptors();
        files = getPlan().getClassFileArray();
        jar = openJarStream(getEarPlan().getArchivePath());
        addModules();
        addLibraries();
        for (int i = 0; i < dd.length; i++) {
            addDescriptor(dd[i], root, files);
            if (dd[i].getArchivePath().equalsIgnoreCase(APPLICATION_XML)) {
                userAppDD = true;
            }
        }
        if (!userAppDD) {
            addApplicationXML();
        }
        closeJarStream();
        cleanTmp();
        return jar;
    }

    protected Manifest buildManifest(Manifest m) {
        Manifest         manifest = super.buildManifest(m);
        Boolean          b = null;
        String[]         eMods = new String[0];
        Descriptor[]     metaDD = new Descriptor[0];
        WebApplication[] webApps = new WebApplication[0];

        // app name
        manifest.getMainAttributes().put(new Attributes.Name(APP_NAME),
                                         getEarPlan().getAppName());

        // description
        manifest.getMainAttributes().put(new Attributes.Name(APP_DESC),
                                         getEarPlan().getDescription());

        // copy client
        b = new Boolean(getEarPlan().isCopyClient());
        manifest.getMainAttributes().put(new Attributes.Name(COPY_CLIENT),
                                         b.toString());

        // alt dd
        b = new Boolean(getEarPlan().isAlt());
        manifest.getMainAttributes().put(new Attributes.Name(ALT_DD),
                                         b.toString());

        // enterprise modules
        eMods = getEarPlan().getEnterpriseModules();
        for (int i = 0; i < eMods.length; i++) {
            manifest.getMainAttributes().put(new Attributes.Name(ENTERPRISE_MODULE
                    + i), eMods[i]);
        }

        // web apps
        webApps = getEarPlan().getWebApplications();
        for (int i = 0; i < webApps.length; i++) {
            manifest.getMainAttributes().put(new Attributes.Name(WEBAPP_MODULE
                    + i), webApps[i].getArchive());
            manifest.getMainAttributes().put(new Attributes.Name(WEBAPP_CONTEXT
                    + i), webApps[i].getContextRoot());
        }

        // meta dd
        metaDD = getEarPlan().getDescriptors();
        for (int i = 0; i < metaDD.length; i++) {
            manifest.getMainAttributes().put(new Attributes.Name(APP_META
                    + i), metaDD[i].getPath());
        }
        return manifest;
    }

    private void addModules() throws ArchiveException {
        Module[] mods = getEarPlan().getAllModules();

        if (getEarPlan().isCopyClient()) {
            mods = copyClients(mods);
        }
        for (int i = 0; i < mods.length; i++) {
            addModule(mods[i]);
        }
    }

    private void addLibraries() throws ArchiveException {
        File[] files = getEarPlan().getLibFileArray();

        for (int i = 0; i < files.length; i++) {
            addLibFile(files[i]);
        }
    }

    private void extractModules(Module mod) throws ArchiveException {
        JarFile     jar = null;
        JarEntry    entry = null;
        InputStream stream = null;
        Enumeration entries = null;
        ArrayList   list = null;

        try {
            jar = new JarFile(new File(mod.getArchive()));
            entries = jar.entries();
            while (entries.hasMoreElements()) {
                entry = (JarEntry) entries.nextElement();
                if (isJar(entry.getName())) {
                    getEarPlan().addEnterpriseModule(entry.getName());
                    stream = jar.getInputStream(entry);
                    createEntry(stream, entry.getName());
                    stream = jar.getInputStream(entry);
                    extractDD(new JarInputStream(stream), entry.getName());
                } else if (isWar(entry.getName())) {
                    getEarPlan().addWebApplication(entry.getName());
                    stream = jar.getInputStream(entry);
                    createEntry(stream, entry.getName());
                    stream = jar.getInputStream(entry);
                    extractDD(new JarInputStream(stream), entry.getName());
                }
            }
            entry = jar.getJarEntry(APPLICATION_XML);
            list = new ArrayList(Arrays.asList(mergeDescriptors));
            list.add(jar.getInputStream(entry));
            list.trimToSize();
            mergeDescriptors = new InputStream[list.size()];
            mergeDescriptors = (InputStream[]) list.toArray(mergeDescriptors);
            list.clear();
        } catch (IOException e) {
            throw new ArchiveException(e,
                                       "Unable to extract modules from: "
                                       + mod.getName());
        }
    }

    private void extractDD(JarInputStream stream,
                           String jarName) throws ArchiveException {
        JarEntry              entry = null;
        String                dd = new String();
        StringBuffer          entryPath = new StringBuffer();
        ByteArrayOutputStream outStream = null;

        //
        if (isWar(jarName)) {
            dd = Descriptor.getArchivePath(Descriptor.WEB);
        } else {
            dd = Descriptor.getArchivePath(Descriptor.EJB);
        }
        entryPath.append(jarName);
        entryPath.append('/');
        entryPath.append(dd);
        try {
            entry = stream.getNextJarEntry();
            while (entry != null) {
                if (entry.getName().equalsIgnoreCase(dd)) {
                    byte[] bytes = new byte[0];

                    outStream = new ByteArrayOutputStream();
                    FileUtil.copy(stream, outStream);
                    bytes = outStream.toByteArray();
                    outStream.close();
                    createEntry(new ByteArrayInputStream(bytes),
                                entryPath.toString());
                    break;
                }
                entry = stream.getNextJarEntry();
            }
            stream.close();
        } catch (IOException e) {
            throw new ArchiveException(e,
                                       "Unable to extract deployment descriptor from: "
                                       + jarName);
        }
    }

    private void extractDD(Module mod) throws ArchiveException {
        JarFile      jar = null;
        JarEntry     entry = null;
        InputStream  stream = null;
        String       dd = new String();
        StringBuffer entryPath = new StringBuffer();

        //
        if (isWar(mod.getName())) {
            dd = Descriptor.getArchivePath(Descriptor.WEB);
        } else {
            dd = Descriptor.getArchivePath(Descriptor.EJB);
        }
        entryPath.append(mod.getName());
        entryPath.append('/');
        entryPath.append(dd);
        try {
            jar = new JarFile(mod.getArchive());
            entry = jar.getJarEntry(dd);
            stream = jar.getInputStream(entry);
            createEntry(stream, entryPath.toString());
        } catch (NullPointerException e) {
            throw new ArchiveException(e,
                                       "Unable to extract deployment descriptor from: "
                                       + mod.getName());
        } catch (IOException e) {
            throw new ArchiveException(e,
                                       "Unable to extract deployment descriptor from: "
                                       + mod.getName());
        }
    }

    private boolean isWar(String path) {
        return path.toString().toLowerCase().endsWith(".war");
    }

    private boolean isJar(String path) {
        return path.toString().toLowerCase().endsWith(".jar");
    }

    private boolean isEar(String path) {
        return path.toString().toLowerCase().endsWith(".ear");
    }

    private void addModule(Module mod) throws ArchiveException {
        String entryPath = new String();

        //
        entryPath = mod.getName();
        if (getEarPlan().isAlt()) {
            if (isWar(entryPath)) {
                extractDD(mod);
            } else if (isJar(entryPath)) {
                extractDD(mod);
            } else if (isEar(entryPath)) {
                extractModules(mod);
            }
        }
        if (!isEar(entryPath)) {

            //
            try {
                createEntry(new File(mod.getArchive()), entryPath);
            } catch (IOException e) {
                String exMess = res.getString("EarBuilder_Unable_to");

                exMess = ResUtil.format(exMess, mod.getName());
                throw new ArchiveException(e, exMess);
            }
        }
    }

    private void addLibFile(File source) throws ArchiveException {
        StringBuffer entryPath = new StringBuffer();

        //
        entryPath.append("lib");
        entryPath.append('/');
        entryPath.append(source.getName());

        //
        try {
            createEntry(source, entryPath.toString());
        } catch (IOException e) {
            String exMess = res.getString("EarBuilder_Unable_to");

            exMess = ResUtil.format(exMess, source);
            throw new ArchiveException(e, exMess);
        }
    }

    private void addApplicationXML() throws ArchiveException {
        EarDescriptorHandler prep = new EarDescriptorHandler();
        StringBuffer         entryPath = new StringBuffer();
        String               classRoot = null;
        String               relative = null;

        prep.setAppName(getEarPlan().getAppName());
        prep.setDescription(getEarPlan().getDescription());
        prep.setAllModules(getEarPlan().getAllModules());
        prep.setAlt(getEarPlan().isAlt());
        prep.setMergeDescriptors(mergeDescriptors);
        prep.prep();
        entryPath.append(APPLICATION_XML);
        try {
            createEntry(prep.getInputStream(), entryPath.toString());
        } catch (IOException e) {
            throw new ArchiveException(e,
                                       ResUtil.format(res.getString("EjbBuilder_Unable_to2"),
                                                      APPLICATION_XML));
        }
    }

    private Module[] copyClients(Module[] mods) {
        ArrayList  list = new ArrayList();
        String[]   clients = new String[0];
        PathHandle ph = null;
//        EJBClient  task = new EJBClient();

        try {
            tmpDir = getTempDir();

            // create clients
            for (int i = 0; i < mods.length; i++) {
                ph = PathHandle.createPathHandle(mods[i].getArchive());
                if (ph.isFile() && ph.hasExtension("jar")) {
//                    task.setJarname(ph.getPath());
                    ph = createTempClientPH(ph);
//                    task.setClientjarname(ph.getPath());
//                    task.execute();
                    if (ph.isFile()) {
                        list.add(ph.getPath());
                    }
                }
            }
            list.trimToSize();
            clients = new String[list.size()];
            clients = (String[]) list.toArray(clients);
            list.clear();
            if (clients.length > 0) {

                // add clients to wars
                for (int i = 0; i < mods.length; i++) {
                    if (mods[i] instanceof WebApplication) {
                        ph = addClients(mods[i], clients);
                        if (ph.isFile()) {
                            mods[i].setArchive(ph.getPath());
                        }
                    }
                }
            }
//            task = null;
            ph = null;

            // delete clients
            for (int i = 0; i < clients.length; i++) {
                File client = new File(clients[i]);

                clients[i] = null;
                if (client.isFile()) {
                    sleep(10);
                    client.deleteOnExit();
                    client.delete();
                    client = null;
                }
            }
        } catch (IOException e) {
            e.printStackTrace(System.err);
        }
        return mods;
    }

    private PathHandle addClients(Module mod,
                                  String[] clients) throws IOException {
        Enumeration     entries = null;
        PathHandle      ph = null;
        JarFile         source = null;
        JarOutputStream out = null;
        File            dest = null;
        Manifest        manifest = null;

        ph = PathHandle.createPathHandle(mod.getArchive());
        dest = File.createTempFile("tmp", ".kelp", tmpDir.getFile());
        source = new JarFile(ph.getFile());
        manifest = source.getManifest();
        out = new JarOutputStream(new FileOutputStream(dest));
        entries = source.entries();
        while (entries.hasMoreElements()) {
            JarEntry entry = null;

            entry = (JarEntry) entries.nextElement();
            out.putNextEntry(entry);
            FileUtil.copy(source.getInputStream(entry), out);
            out.flush();
            out.closeEntry();
        }
        for (int i = 0; i < clients.length; i++) {
            JarEntry        entry = null;
            FileInputStream in = null;

            ph = PathHandle.createPathHandle(clients[i]);
            in = new FileInputStream(ph.getFile());
            ph.setExtension("jar");
            entry = new JarEntry("lib/" + ph.getFile().getName());
            entry.setTime(System.currentTimeMillis());
            out.putNextEntry(entry);
            FileUtil.copy(in, out);
            in.close();
            out.flush();
            out.closeEntry();
            in = null;
        }
        source.close();
        out.flush();
        out.close();
        return PathHandle.createPathHandle(dest);
    }

    private PathHandle getTempDir() throws IOException {
        File tmp = null;

        tmp = File.createTempFile("tmp", ".kelp");
        return PathHandle.createPathHandle(tmp.getParentFile());
    }

    private PathHandle createTempClientPH(PathHandle ejb) throws IOException {
        File       tmpFile = null;
        PathHandle ph = null;
        String     tmpName = ejb.getFile().getName();

        tmpName = tmpName.substring(0, tmpName.indexOf('.'));
        tmpName = tmpName + "Client";
        tmpFile = File.createTempFile(tmpName, ".kelp", tmpDir.getFile());
        ph = PathHandle.createPathHandle(tmpFile);
        tmpFile.delete();
        return ph;
    }

    private void cleanTmp() {
        PathHandle ph;

        if (tmpDir != null) {
            sleep(10);
            File[] files = tmpDir.getFile().listFiles();

            sleep(10);
            for (int i = 0; i < files.length; i++) {
                ph = PathHandle.createPathHandle(files[i]);
                if (ph.hasExtension("kelp")) {
                    ph.getFile().deleteOnExit();
                    ph.getFile().delete();
                }
                sleep(10);
            }
            sleep(10);
            tmpDir = null;
        }
    }

    private void sleep(int m) {
        try {
            Thread.sleep(m);
        } catch (InterruptedException e) {
            e.printStackTrace(System.err);
        }
    }

}
