/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive;

// ToolBox
import org.enhydra.tool.common.PathHandle;

// JDK
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.Properties;

//
public class PropertyReader implements PropertyKeys {
    private Properties props = new Properties();
    private File       file = null;
    private boolean    validate = true;

    public PropertyReader() {}

    public boolean isValidate() {
        return validate;
    }

    public void setValidate(boolean v) {
        validate = v;
    }

    public JarPlan[] read(File in) throws IOException {
        FileInputStream     fileStream = null;
        BufferedInputStream bufStream = null;

        file = in;
        fileStream = new FileInputStream(file);
        bufStream = new BufferedInputStream(fileStream);
        props.load(bufStream);
        bufStream.close();
        fileStream.close();
        return read(props);
    }

    public JarPlan[] read(Properties p) {
        ArrayList list = null;
        JarPlan[] plans = new JarPlan[0];

        props = p;
        list = new ArrayList();
        for (int i = 0; i < 100; i++) {
            JarPlan cursor = null;
            String  value = null;

            value = pathValue(props.getProperty(PATH + i));
            if (isEmpty(value)) {
                break;
            } else {
                try {
                    cursor = createPlan(i);

                    // TODO: relative root expansion
                    cursor.setArchivePath(value);
                } catch (ArchiveException e) {
                    e.printStackTrace(System.err);
                    cursor = null;
                }
            }
            if (cursor != null) {
                list.add(cursor);
            }
        }
        list.trimToSize();
        plans = new JarPlan[list.size()];
        plans = (JarPlan[]) list.toArray(plans);
        list.clear();
        return plans;
    }

    private JarPlan createPlan(int count) throws ArchiveException {
        String  warValue = null;
        String  earValue = null;
        String  ejbValue = null;
        String  jarValue = null;
        String  serviceValue = null;
        JarPlan plan = null;

        earValue = props.getProperty(APP_NAME_PROP + count);
        warValue = props.getProperty(CONTENT_ROOT_PROP + count);
        ejbValue = props.getProperty(CREATE_CLIENT_PROP + count);
        serviceValue = props.getProperty(Descriptor.ENHYDRA_SERVICE + '.'
                                         + count);
        jarValue = props.getProperty(CLASS_ROOT_PROP + count);
        if (!isEmpty(earValue)) {
            plan = createEarPlan(count);
        } else if (!isEmpty(warValue)) {
            plan = createWarPlan(count);
        } else if (!isEmpty(ejbValue)) {
            plan = createEjbPlan(count);
        } else if (!isEmpty(serviceValue)) {
            plan = createServicePlan(count);
        } else if (!isEmpty(jarValue)) {
            plan = createJarPlan(count);
        }
        return plan;
    }

    private JarPlan createJarPlan(int count) throws ArchiveException {
        JarPlan plan = null;

        plan = new JarPlan();
        readJarProperties(plan, count);
        return plan;
    }

    private ServicePlan createServicePlan(int count) throws ArchiveException {
        ServicePlan plan = null;

        plan = new ServicePlan();
        readJarProperties(plan, count);
        return plan;
    }

    private void readJarProperties(JarPlan plan,
                                   int count) throws ArchiveException {
        Descriptor[] dd = new Descriptor[0];
        String       value = null;
        String[]     files = new String[0];

        plan.setValidate(isValidate());

        // classes
        value = pathValue(props.getProperty(CLASS_ROOT_PROP + count));
        plan.setClassRoot(value);
        files = readFiles(CLASS, count);
        if (files.length > 0) {
            plan.setClassFiles(files);
        }

        // dd
        dd = plan.getDescriptors();
        for (int i = 0; i < dd.length; i++) {
            value = pathValue(props.getProperty(dd[i].getType() + "."
                                                + count));
            dd[i].setPath(value);
        }
        plan.setDescriptors(dd);
    }

    private EjbPlan createEjbPlan(int count) throws ArchiveException {
        EjbPlan plan = null;
        Boolean b = null;
        String  value = null;

        plan = new EjbPlan();
        readJarProperties(plan, count);

        // create client
        value = props.getProperty(CREATE_CLIENT_PROP + count);
        b = Boolean.valueOf(value);
        plan.setCreateClient(b.booleanValue());
        return plan;
    }

    private EarPlan createEarPlan(int count) throws ArchiveException {
        EarPlan          plan = null;
        String           value = null;
        Boolean          b = null;
        String[]         files = new String[0];
        WebApplication[] webApps = new WebApplication[0];
        Descriptor[]     dd = new Descriptor[0];

        plan = new EarPlan();
        plan.setValidate(isValidate());

        // app name
        value = props.getProperty(APP_NAME_PROP + count);
        plan.setAppName(value);

        // description
        value = props.getProperty(APP_DESC_PROP + count);
        plan.setDescription(value);

        // alt-dd
        value = props.getProperty(ALT_DD_PROP + count);
        b = Boolean.valueOf(value);
        plan.setAlt(b.booleanValue());

        // copy-client
        value = props.getProperty(COPY_CLIENT_PROP + count);
        b = Boolean.valueOf(value);
        plan.setCopyClient(b.booleanValue());

        // enterprise modules
        files = readFiles(ENTERPRISE_MODULE_PROP, count);
        if (files.length > 0) {
            plan.setEnterpriseModules(files);
        }

        // webapps
        webApps = readWebApps(count);
        if (webApps.length > 0) {
            plan.setWebApplications(webApps);
        }

        // libs
        files = readFiles(LIBRARY_PROP, count);
        if (files.length > 0) {
            plan.setLibFiles(files);
        }

        // meta-dd
        files = readFiles(META_DD_PROP, count);
        if (files.length > 0) {
            dd = new Descriptor[files.length];
            for (int i = 0; i < dd.length; i++) {
                dd[i] = new Descriptor(files[i]);
            }
            plan.setDescriptors(dd);
        }
        return plan;
    }

    private WarPlan createWarPlan(int count) throws ArchiveException {
        WarPlan      plan = null;
        String       value = null;
        String[]     files = new String[0];
        Descriptor[] dd = new Descriptor[0];

        plan = new WarPlan();
        readJarProperties(plan, count);

        // content
        value = pathValue(props.getProperty(CONTENT_ROOT_PROP + count));
        plan.setContentRoot(value);
        files = readFiles(CONTENT, count);
        if (files.length > 0) {
            plan.setContentFiles(files);
        }

        // libs
        files = readFiles(LIBRARY_PROP, count);
        if (files.length > 0) {
            plan.setLibFiles(files);
        }

        //
        return plan;
    }

    private String[] readFiles(String property, int count) {
        ArrayList list = new ArrayList();
        String[]  files = new String[0];
        String    value = null;

        for (int i = 0; i < 2500; i++) {
            value = pathValue(props.getProperty(property + count + "." + i));
            if (isEmpty(value)) {
                break;
            } else {
                list.add(value);
            }
        }
        list.trimToSize();
        files = new String[list.size()];
        files = (String[]) list.toArray(files);
        return files;
    }

    private WebApplication[] readWebApps(int count) {
        ArrayList        list = new ArrayList();
        WebApplication[] webApps = new WebApplication[0];
        String           value = null;

        for (int i = 0; i < 2500; i++) {
            value = pathValue(props.getProperty(WEBAPP_MODULE_PROP + count
                                                + "." + i));
            if (isEmpty(value)) {
                break;
            } else {
                WebApplication webApp = new WebApplication(value);

                value = props.getProperty(WEBAPP_CONTEXT_PROP + count + "."
                                          + i);
                if (!isEmpty(value)) {
                    webApp.setContextRoot(value);
                }
                list.add(webApp);
            }
        }
        list.trimToSize();
        webApps = new WebApplication[list.size()];
        webApps = (WebApplication[]) list.toArray(webApps);
        return webApps;
    }

    private boolean isEmpty(String value) {
        boolean empty = false;

        if (value == null) {
            empty = true;
        } else if (value.trim().length() == 0) {
            empty = true;
        }
        return empty;
    }

    private String pathValue(String in) {
        String out = null;

        if (file == null) {
            out = in;
        } else if (isEmpty(in)) {
            out = in;
        } else {
            PathHandle propPh = null;
            PathHandle outPh = null;

            propPh = PathHandle.createPathHandle(file);
            outPh = propPh.expandRelativePath(in);
            out = outPh.getPath();
        }
        return out;
    }

}
