/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

// ToolBox
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.archive.EarPlan;

// JDK
import java.awt.*;
import java.io.File;
import javax.swing.*;
import java.beans.*;
import javax.swing.border.*;

//
public class EarFilePanel extends ArchivePanel {
    private FilePanel     filePanel = null;
    private GridBagLayout layoutMain = null;
    private JPanel        panelApp = null;
    private GridBagLayout layoutApp = null;
    private JLabel        labelAppName = null;
    private JTextField    textAppName = null;
    private JLabel        labelAppDesc = null;
    private JScrollPane   scroll = null;
    private JTextArea     textAppDesc = null;
    private JCheckBox     checkAlt = null;
    private JCheckBox     checkCopyClient = null;
    private JPanel        panellFiller = null;

    public EarFilePanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    // override ArchivePanel
    protected void setWizard(ArchiveWizard wizard) {
        super.setWizard(wizard);
        filePanel.setWizard(wizard);
    }

    public void readPlan(JarPlan plan) throws ArchiveException {
        filePanel.setWizard(getWizard());
        filePanel.setArchivePath(plan.getArchivePath());
        filePanel.getDDTableModel().clear();
        filePanel.getDDTable().setVisible(false);
        if (plan instanceof EarPlan) {
            EarPlan ep = (EarPlan) plan;

            checkCopyClient.setSelected(ep.isCopyClient());
            checkAlt.setSelected(ep.isAlt());
            textAppName.setText(ep.getAppName());
            textAppDesc.setText(ep.getDescription());
        } else {
            throw new ArchiveException("Unable to read plan: expecting type EarPlan");
        }
    }

    public void writePlan(JarPlan plan) throws ArchiveException {
        validatePlan();
        plan.setArchivePath(filePanel.getArchivePath());
        if (plan instanceof EarPlan) {
            EarPlan ep = (EarPlan) plan;

            ep.setCopyClient(checkCopyClient.isSelected());
            ep.setAlt(checkAlt.isSelected());
            ep.setAppName(textAppName.getText());
            ep.setDescription(textAppDesc.getText());
        } else {
            throw new ArchiveException("Unable to write plan: expecting type EarPlan");
        }
    }

    public void validatePlan() throws ArchiveException {
        filePanel.validatePlan();
        if (textAppName.getText().trim().length() == 0) {
            throw new ArchiveException("Application name cannot be left blank.");
        }
    }

    public String getPageTitle() {
        return "Enterprise Application Archive";
    }

    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append("Enter details for the enterprise application archive file.");
        return buf.toString();
    }

    protected void preparePanel() {
        String[]     defaultDD = new String[1];
        StringBuffer buf = new StringBuffer();

        buf.append(getWorkingRoot());
        buf.append(File.separator);
        buf.append("src");
        buf.append(File.separator);
        buf.append("meta-inf");
        buf.append(File.separator);
        buf.append("application.xml");
        defaultDD[0] = buf.toString();
        filePanel.setDefaultDD(defaultDD);
        filePanel.preparePanel();
    }

    private void pmInit() {
        filePanel.setExtension("ear");
        checkAlt.setEnabled(false);
        checkAlt.setSelected(false);
    }

    private void jbInit() throws Exception {
        filePanel = (FilePanel) Beans.instantiate(getClass().getClassLoader(),
                                                  FilePanel.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelApp = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                              JPanel.class.getName());
        layoutApp =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        labelAppName = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                  JLabel.class.getName());
        textAppName =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        labelAppDesc = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                  JLabel.class.getName());
        scroll = (JScrollPane) Beans.instantiate(getClass().getClassLoader(),
                                                 JScrollPane.class.getName());
        textAppDesc =
            (JTextArea) Beans.instantiate(getClass().getClassLoader(),
                                          JTextArea.class.getName());
        checkAlt = (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                                 JCheckBox.class.getName());
        checkCopyClient =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        panellFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  JPanel.class.getName());
        panelApp.setLayout(layoutApp);
        panelApp.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(new Color(187, 218, 252), new Color(91, 107, 123)),
                                            "Application"));
        labelAppName.setText("Name:");
        textAppName.setText("APPLICATION NAME");
        labelAppDesc.setText("Description:");
        textAppDesc.setText("Application description");
        checkAlt.setText("Create alternate descriptors");
        checkCopyClient.setEnabled(true);
        checkCopyClient.setSelected(true);
        checkCopyClient.setText("Copy EJB clients to web archives");
        panelApp.add(labelAppName,
                     new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                            GridBagConstraints.WEST,
                                            GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 10, 0, 5), 0, 0));
        panelApp.add(textAppName,
                     new GridBagConstraints(1, 0, 1, 1, 0.1, 0.1,
                                            GridBagConstraints.WEST,
                                            GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 5, 0, 10), 0, 0));
        panelApp.add(labelAppDesc,
                     new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                            GridBagConstraints.WEST,
                                            GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 10, 5, 5), 0, 0));
        panelApp.add(scroll,
                     new GridBagConstraints(1, 1, 1, 3, 0.4, 0.6,
                                            GridBagConstraints.CENTER,
                                            GridBagConstraints.BOTH,
                                            new Insets(5, 5, 5, 10), 0, 50));
        panelApp.add(checkAlt,
                     new GridBagConstraints(0, 4, 2, 1, 0.1, 0.1,
                                            GridBagConstraints.WEST,
                                            GridBagConstraints.HORIZONTAL,
                                            new Insets(2, 5, 2, 5), 0, 0));
        panelApp.add(checkCopyClient,
                     new GridBagConstraints(0, 5, 2, 1, 0.0, 0.0,
                                            GridBagConstraints.CENTER,
                                            GridBagConstraints.HORIZONTAL,
                                            new Insets(2, 5, 2, 5), 0, 0));
        panelApp.add(panellFiller,
                     new GridBagConstraints(0, 2, 1, 1, 0.1, 0.1,
                                            GridBagConstraints.CENTER,
                                            GridBagConstraints.NONE,
                                            new Insets(0, 0, 0, 0), 5, 5));
        scroll.getViewport().add(textAppDesc, null);
        this.setLayout(layoutMain);
        this.add(filePanel,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 0, 0, 0), 0, 0));
        this.add(panelApp,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(2, 5, 5, 5), 0, 0));
    }

}
