/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

// ToolBox
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.Descriptor;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.archive.EjbPlan;

// JDK
import java.awt.*;
import java.io.File;
import javax.swing.*;
import java.lang.ref.WeakReference;
import java.beans.*;

//
public class EjbFilePanel extends ArchivePanel {
    private FilePanel     filePanel = null;
    private JCheckBox     checkCreateClient = null;
    private GridBagLayout layoutMain = null;

    public EjbFilePanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void readPlan(JarPlan plan) throws ArchiveException {
        filePanel.setArchivePath(plan.getArchivePath());
        if (plan instanceof EjbPlan) {
            EjbPlan ep = (EjbPlan) plan;
            Descriptor[] row = plan.getDescriptors();
            filePanel.getDDTableModel().clear();
            for (int i = 0 ; i < row.length ; i++ ) {
              filePanel.getDDTableModel().addDescriptor(row[i]);
            }
            filePanel.getDDTable().setRowSelectionInterval(0, 0);
            checkCreateClient.setSelected(ep.isCreateClient());
        } else {
            throw new ArchiveException("Unable to read plan: expecting type EjbPlan");
        }
    }

    public void writePlan(JarPlan plan) throws ArchiveException {
        validatePlan();
        plan.setArchivePath(filePanel.getArchivePath());
        if (plan instanceof EjbPlan) {
            EjbPlan ep = (EjbPlan) plan;
            int     count = filePanel.getDDTableModel().getRowCount();

            ep.setDescriptors(filePanel.getDDTableModel().getDescriptors());
            ep.setCreateClient(checkCreateClient.isSelected());
        }
    }

    public void validatePlan() throws ArchiveException {
        filePanel.validatePlan();
    }

    public String getPageTitle() {
        return "Enterprise JavaBean Archive";
    }

    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append("Enter details for the enterprise JavaBean archive.");
        return buf.toString();
    }

    protected void preparePanel() {
        String[]     defaultDD = new String[1];
        StringBuffer buf = new StringBuffer();

        buf.append(getWorkingRoot());
        buf.append(File.separator);
        buf.append("src");
        buf.append(File.separator);
        buf.append("meta-inf");
        buf.append(File.separator);
        buf.append("ejb-jar.xml");
        defaultDD[0] = buf.toString();
        filePanel.setDefaultDD(defaultDD);
        filePanel.preparePanel();
    }

    private void pmInit() {
        filePanel.setExtension("jar");
    }

    // override ArchivePanel
    protected void setWizard(ArchiveWizard wizard) {
      super.setWizard(wizard);
      filePanel.setWizard(wizard);
    }

    private void jbInit() throws Exception {
        filePanel = (FilePanel) Beans.instantiate(getClass().getClassLoader(),
                                                  FilePanel.class.getName());
        checkCreateClient =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        checkCreateClient.setText("Create client jar");
        this.setLayout(layoutMain);
        this.add(filePanel,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 0, 0, 0), 0, 0));
        this.add(checkCreateClient,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(0, 10, 5, 5), 0, 0));
    }

}
