/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

// ToolBox
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.Descriptor;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.archive.EjbPlan;

// JDK
import java.awt.*;
import java.io.File;
import javax.swing.*;
import java.lang.ref.WeakReference;
import java.beans.*;

//
public class ServiceFilePanel extends ArchivePanel {
    private FilePanel     filePanel = null;
    private GridBagLayout layoutMain = null;

    public ServiceFilePanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void readPlan(JarPlan plan) throws ArchiveException {
    	Descriptor[] rows = new Descriptor[0];
        filePanel.setArchivePath(plan.getArchivePath());
        rows = plan.getDescriptors();
        filePanel.getDDTableModel().clear();
        for (int i = 0 ; i < rows.length ; i++ ) {
           filePanel.getDDTableModel().addDescriptor(rows[i]);
        }
        if (rows.length > 0) {
          filePanel.getDDTable().setRowSelectionInterval(0, 0);
        }
    }

    public void writePlan(JarPlan plan) throws ArchiveException {
    	int  count = 0;
        validatePlan();
        count = filePanel.getDDTableModel().getRowCount();
        plan.setArchivePath(filePanel.getArchivePath());
        plan.setDescriptors(filePanel.getDDTableModel().getDescriptors());
    }

    public void validatePlan() throws ArchiveException {
        filePanel.validatePlan();
    }

    public String getPageTitle() {
        return "Enhydra Service";
    }

    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append("Enter details for the Enhydra service archive.");
        return buf.toString();
    }

    protected void preparePanel() {
        String[]     defaultDD = new String[1];
        StringBuffer buf = new StringBuffer();

        buf.append(getWorkingRoot());
        buf.append(File.separator);
        buf.append("src");
        buf.append(File.separator);
        buf.append("META-INF");
        buf.append(File.separator);
        buf.append("enhydra-service.xml");
        defaultDD[0] = buf.toString();
        filePanel.setDefaultDD(defaultDD);
        filePanel.preparePanel();
    }

    private void pmInit() {
        filePanel.setExtension("jar");
    }

    // override ArchivePanel
    protected void setWizard(ArchiveWizard wizard) {
      super.setWizard(wizard);
      filePanel.setWizard(wizard);
    }

    private void jbInit() throws Exception {
        filePanel = (FilePanel) Beans.instantiate(getClass().getClassLoader(),
                                                  FilePanel.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        this.setLayout(layoutMain);
        this.add(filePanel,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 0, 0, 0), 0, 0));
    }

}
