/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.codegen;
//
import java.io.File;
import java.util.StringTokenizer;

/**
 * This class instantiates a set of default GeneratorOptions during
 * initialization. The default options include: appname, d, package
 * copywrite, copywritefile, nomake, noscript, and nojbuilder.
 */
public class ValidationUtil {

    /**
     * Hidden constructor
     */
    private ValidationUtil() {}

    /**
     * Test to see if the given package is a valid java package name.
     *
     * @param pack
     * The package name to test
     *
     * @return
     * True if the specified package name is valid.
     */
    public static boolean isJavaPackage(String pack) {
        final String DOT = (new String()) + '.';

        boolean         valid = true;
        StringTokenizer tokenizer = null;

        if (pack.trim().length() == 0) {
          valid = false;
        } else if (pack.trim().startsWith(DOT)
                || pack.trim().endsWith(DOT)) {
            valid = false;
        } else {
            tokenizer = new StringTokenizer(pack, DOT);
            while (tokenizer.hasMoreTokens()) {
                if (!ValidationUtil.isJavaIdentifier(tokenizer.nextToken())) {
                    valid = false;
                    break;
                }
            }
        }
        return valid;
    }

    /**
     * Test to see if the given indentifier is a valid java identifier.
     *
     * @param ident
     * The identifier to test.
     *
     * @return
     * True if the specified identifier is valid.
     */
    public static boolean isJavaIdentifier(String ident) {
        boolean valid = true;

        if (ident == null || ident.trim().length() == 0) {
          valid = false;
        } else {
          for (int i = 0; i < ident.length(); i++) {
              char ch = ident.charAt(i);

              if (i == 0) {
                  if (!Character.isJavaIdentifierStart(ch)) {
                      valid = false;
                      break;
                  }
              } else if (!Character.isJavaIdentifierPart(ch)) {
                  valid = false;
                  break;
              }
           }
        }
        return valid;
    }

    /**
     * Test to see if a path is a directory.
     *
     * @param path
     * The path to test.
     *
     * @return
     * True if the specified path is a directory.
     */
    public static boolean isDirectory(String path) {
        boolean valid = false;
        File    file = new File(path);

        if (file != null) {
            valid = file.isDirectory();
        }
        return valid;
    }

    /**
     * Test to see if the parent of a given path is a valid directory.
     *
     * @param path
     * The path to test.
     *
     * @return
     * True if the parent of the specified path is a directory.
     */
    public static boolean isParentDirectory(String path) {
        boolean valid = false;
        File    file = new File(path);

        if (file != null && file.getParentFile() != null) {
            valid = file.getParentFile().isDirectory();
        }
        return valid;
    }

}
