/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.codegen.internal;

// Toolbox imports
import org.enhydra.tool.codegen.GeneratorException;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.Replacement;
import org.enhydra.tool.common.ToolException;

// Standard imports
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;

/**
 * A generator for Servlet 2.2 style Web Applications.
 */
public class WebAppGenerator extends AppGenerator {

    /**
     * Name to use specify at the command line.
     */
    public static final String COMMAND_NAME = "webapp";    // nores

    /**
     * Descriptive name for generator.
     */
    public static final String displayName = res.getString("Web_Application");
    public static final String wizardTitle =
        res.getString("Web_Application1");

    /**
     * Create an instance of the Web Application generator.
     */
    public WebAppGenerator() {
      super();
    }

    /**
     * Implements AppGenerator
     */
    public String[] getOutputExcludes() {
        String[]  ex = new String[0];
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(super.getOutputExcludes()));

        // not to be resourced
        if (getClient().equalsIgnoreCase("chtml")               // nores
        || getClient().equalsIgnoreCase("wml")) {               // nores
            list.add("index.jsp");                              // nores
            list.add("RedirectServlet.java");                   // nores
            list.add("options.xmlc");                           // nores
            list.add("media" + File.separator + "makefile");    // nores
            list.add(".gif");                                   // nores
        }
        list.trimToSize();
        ex = new String[list.size()];
        ex = (String[]) list.toArray(ex);
        list.clear();
        return ex;
    }

    /**
     * Overrides AppGenerator
     */
    public Replacement[] createReplacementsForDirectory(PathHandle path)
            throws GeneratorException {
        Replacement[] reps = new Replacement[0];

        reps = super.createReplacementsForDirectory(path);
        if (path.endsWith("/presentation")) {                       // nores
            ArrayList   list = null;
            String[]    targets = new String[1];
            String[]    rules = new String[1];
            Replacement targetRep = null;
            Replacement ruleRep = null;

            targets[0] = new String();
            rules[0] = new String();
            list = new ArrayList(Arrays.asList(reps));
            if (getClient().equalsIgnoreCase("chtml")) {            // nores

                // defaults OK
            } else if (getClient().equalsIgnoreCase("wml")) {       // nores
                targets = new String[2];
                rules = new String[1];
                targets[0] = new String();
                targets[1] = "XMLC_" + getClient().toUpperCase()    // nores
                + "_OPTS_FILE = options.xmlc";                      // nores
                rules[0] = new String();
            } else {

                // HTML 4 Browser
                targets = new String[5];
                rules = new String[4];
                targets[0] = "RedirectServlet";                     // nores
                targets[1] = new String();
                targets[2] = "SUBDIRS = media";                     // nores
                targets[3] = "XMLC_" + getClient().toUpperCase()    // nores
                + "_OPTS_FILE = options.xmlc";                      // nores
                targets[4] = "MISC_BUILD = content";                // nores
                rules[0] = new String();
                rules[1] = "content::";                             // nores
                rules[2] = "\t"                                     // nores
                + "cp ../resources/index.jsp ${ROOT}/output/content/index.jsp";    // nores
                rules[3] = new String();
            }
            try {
                targetRep = new Replacement("@PRESENTATION_TARGETS@",    // nores
                targets);
                ruleRep = new Replacement("@PRESENTATION_RULES@",
                                          rules);                   // nores
                list.add(targetRep);
                list.add(ruleRep);
            } catch (ToolException e) {
                throw new GeneratorException(e,
                                             res.getString("Unable_to_create1"));
            }
            reps = new Replacement[list.size()];
            reps = (Replacement[]) list.toArray(reps);
            list.clear();
        }
        return reps;
    }

    /**
     * Get short name used in command line. May also be used
     * to locate template directory.
     *
     * @return
     * Command line identifier.
     */
    public String getCommandName() {
        return COMMAND_NAME;
    }

    /**
     * Name to display in command line messages and in the wizard.
     *
     * @return
     * Display name for generator.
     */
    public String getDisplayName() {
        return WebAppGenerator.displayName;
    }

    public String getWizardTitle() {
        return WebAppGenerator.wizardTitle;
    }

    /**
     * Get text that breifly explains the application that
     * the generator creates.
     *
     * @return
     * Descriptive text to display in a wizard or in a command
     * line help response.
     */
    public String getDescription() {
        return res.getString("WebDescript");
    }

}
