/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */

package org.enhydra.tool.codegen.wizard;

//
// ToolBox imports
//
import org.enhydra.tool.common.wizard.TBWizardDeck;
import org.enhydra.tool.codegen.Generator;
import org.enhydra.tool.codegen.GeneratorException;
import org.enhydra.tool.codegen.ValidationException;

// Standard imports
import java.awt.Component;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class CodeGenDeck extends TBWizardDeck implements RootEditor {
    private Generator  selection = null;
    private boolean    selectionHidden = false;
    private boolean    allowRootEdit = true;

    public CodeGenDeck() {
      super();
    }

    public boolean isAllowRootEdit() {
       return allowRootEdit;
    }

    public void setAllowRootEdit(boolean edit) {
       allowRootEdit = edit;
    }

    protected void setSelectionPageHidden(boolean b) {
        selectionHidden = b && (selection != null);
        if (selectionHidden && getPageIndex() == 0) {
            next();
        }
    }

    protected boolean isSelectionPageHidden() {
        return selectionHidden;
    }

    /**
     * Method declaration
     *
     *
     * @param selection
     */
    protected void setSelection(Generator sel)
            throws GeneratorException {
        selection = sel;
        if (selection == null) {
          System.err.println("GodeGenDeck.setSelect(): null selection");
        } else {
            while (getPages().length > 1) {
                removeWizardPage(getPages()[1]);
            }
            selection.setSwing(true);
            for (int i = 0; i < selection.getWizardPanels().length; i++) {
                CodeGenPage  newPage = new CodeGenPage();
                CodeGenPanel panel = selection.getWizardPanels()[i];
                if (panel instanceof RootEditor) {
                  RootEditor editor = (RootEditor) panel;
                  editor.setAllowRootEdit(isAllowRootEdit());
                }

                newPage.add(panel);
                newPage.setPageTitle(panel.getPageTitle());
                newPage.setInstructions(panel.getInstructions());
                addWizardPage(newPage);
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    protected Generator getSelection() {
        return selection;
    }

    protected boolean isSelectionPageShowing() {
        return getPageIndex() == 0;
    }

    // override TBWizardDeck
    protected boolean isFirstPageShowing() {
        boolean atFirst = super.isFirstPageShowing();

        if (isSelectionPageHidden()) {
            atFirst = (getPageIndex() == 1);
        }
        return (atFirst);
    }

    protected boolean isLastPageShowing() {
        boolean atLast = super.isLastPageShowing();

        if (! isSelectionPageHidden()) {
            if (isSelectionPageShowing()) {
              atLast = false;
            }
        }
        return (atLast);
    }



    /**
     * Method declaration
     *
     *
     * @exception GeneratorException
     * @exception ValidationException
     *
     * @see
     */
    protected void writeOptionSet()
            throws ValidationException, GeneratorException {
        CodeGenPage page = null;

        // validate all pages before writing any
        for (int i = 0; i < getPages().length; i++) {
            page = (CodeGenPage) getPages()[i];
            page.validateOptionSet();
        }
        for (int i = 0; i < getPages().length; i++) {
            page = (CodeGenPage) getPages()[i];
            page.writeOptionSet();
        }
    }

}
