/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.common.wizard;

// ToolBox imports
import org.enhydra.tool.common.ButtonPanel;

// Standard imports
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.*;
import javax.swing.*;

//
public class TBWizardButtonPanel extends ButtonPanel {

    //
    private GridBagLayout       layoutMain = null;
    private JPanel              panelFiller = null;
    private JPanel              panelLine;
    private JButton             buttonBack = null;
    private JButton             buttonNext = null;
    private JButton             buttonFinish = null;
    private JButton             buttonCancel = null;
    private JButton             buttonHelp = null;
    private LocalButtonListener buttonListener = null;

    public final static String[] getOptions() {
        final String[] options = {
            ButtonPanel.COMMAND_BACK, ButtonPanel.COMMAND_NEXT,
            ButtonPanel.COMMAND_FINISH, ButtonPanel.COMMAND_CANCEL,
            ButtonPanel.COMMAND_HELP
        };

        return options;
    }

    public TBWizardButtonPanel() {
        super();
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // override ButtonPanel
    public void removeHelp() {
        remove(buttonHelp);
        invalidate();
        doLayout();
    }

    //
    //
    private void pmInit() {
        buttonListener = new LocalButtonListener();

        //
        buttonBack.addActionListener(buttonListener);
        buttonNext.addActionListener(buttonListener);
        buttonFinish.addActionListener(buttonListener);
        buttonCancel.addActionListener(buttonListener);
        buttonHelp.addActionListener(buttonListener);

        //
        buttonBack.setText(ButtonPanel.COMMAND_BACK);
        buttonNext.setText(ButtonPanel.COMMAND_NEXT);
        buttonFinish.setText(ButtonPanel.COMMAND_FINISH);
        buttonCancel.setText(ButtonPanel.COMMAND_CANCEL);
        buttonHelp.setText(ButtonPanel.COMMAND_HELP);

        //
        buttonBack.setActionCommand(buttonBack.getText());
        buttonNext.setActionCommand(buttonNext.getText());
        buttonFinish.setActionCommand(buttonFinish.getText());
        buttonCancel.setActionCommand(buttonCancel.getText());
        buttonHelp.setActionCommand(buttonHelp.getText());
        buttonBack.setMnemonic('B');
        buttonNext.setMnemonic('N');
        buttonHelp.setMnemonic('H');
        buttonFinish.setEnabled(false);
        buttonBack.setEnabled(false);
    }

    private void jbInit() throws Exception {
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelLine = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        buttonBack = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        buttonNext = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        buttonFinish =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonCancel =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonHelp = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        panelLine.setMinimumSize(new Dimension(5, 5));
        panelLine.setPreferredSize(new Dimension(5, 5));
        panelLine.setBorder(BorderFactory.createEtchedBorder());

        buttonBack.setText("BACK");

        // buttonBack.setEnabled(false);
        buttonNext.setText("NEXT");
        buttonFinish.setText("FINISH");

        // buttonFinish.setEnabled(false);
        buttonCancel.setText("CANCEL");
        buttonHelp.setText("HELP");
        this.setLayout(layoutMain);
        this.add(panelLine, new GridBagConstraints(0, 0, 6, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 1, 1, 1), 0, 0));
         this.add(panelFiller,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 0, 0, 0), 0, 0));
        this.add(buttonBack, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
        this.add(buttonNext, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
        this.add(buttonFinish, new GridBagConstraints(3, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
        this.add(buttonCancel, new GridBagConstraints(4, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
        this.add(buttonHelp, new GridBagConstraints(5, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
    }

    private class LocalButtonListener implements ActionListener {

        /**
         * ActionListener implementation.
         *
         * @param event
         * Event that triggered this listener.
         *
         */
        public void actionPerformed(ActionEvent e) {
            Object      source = null;
            ActionEvent event = null;

            source = e.getSource();
            if (source == buttonBack) {
                event = new ActionEvent(buttonBack, ButtonPanel.BACK,
                                        buttonBack.getActionCommand());
            } else if (source == buttonNext) {
                event = new ActionEvent(buttonNext, ButtonPanel.NEXT,
                                        buttonNext.getActionCommand());
            } else if (source == buttonFinish) {
                event = new ActionEvent(buttonFinish, ButtonPanel.FINISH,
                                        buttonFinish.getActionCommand());
            } else if (source == buttonCancel) {
                event = new ActionEvent(buttonCancel, ButtonPanel.CANCEL,
                                        buttonCancel.getActionCommand());
            } else if (source == buttonHelp) {
                event = new ActionEvent(buttonHelp, ButtonPanel.HELP,
                                        buttonHelp.getActionCommand());
            }
            if (event != null) {
                notifyActionListeners(event);
            }
        }

    }
}
